using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Threading;
using System.Runtime.InteropServices;
using DuffBeer.Communication;
using MrHomer.Networking;
using DuffBeer.Messages.Serialization;
using DuffBeer;
using DuffBeer.Sensors;
using DuffBeer.Messages.MessageTypes;
using DuffBeer.Messages;
using MrHomer.Security;
using DuffBeer.Database.StructureTypes;
using DONUT.FakePlugins;

namespace DONUT
{
    public partial class MainForm : Form
    {
        private struct SensorReading : IComparable
        {
            public double Avg;
            public double Min;
            public double Max;
            public double Total;
            public double Duration;
            public string Time;
            public List<string> Filters;

            #region IComparable Members

            public int CompareTo(object obj)
            {
                if (!(obj is SensorReading))
                    return 0;
                SensorReading r = (SensorReading)obj;
                return Time.CompareTo(r.Time);
            }

            #endregion
        }

        #region Constants

        private const int SystemRefreshRate = 5;
        private const int SensorRefreshRate = 5;

        #endregion

        #region Variables

        //Dictionary<string, Panel> PanelList;
        Dictionary<string, Panel> NavPanelList;
        Dictionary<string, Panel> MainPanelList;
        List<Panel> ActivePanels;

        Dictionary<string, List<SensorReading>> SensorData;

        bool running;
        Thread eve;

        Counter cnt, cnt2;
        int levelsDown;
        int SystemCounter;

        #endregion

        public MainForm()
        {
            InitializeComponent();
            running = true;
            this.Hide();

            levelsDown = 0;
            SystemCounter = -1;
            cnt = new Counter();
            cnt2 = new Counter();
            SensorData = new Dictionary<string, List<SensorReading>>();

            ThreadStart t = new ThreadStart(pain);
            Thread th = new Thread(t);
            th.Name = "Pain";
            th.Start();

#if !DEBUG
            Thread.Sleep(5000);
#else
            Thread.Sleep(1000);
#endif

#if DEBUG
            SensorManager.getInstance().InstantiateSensor("Photovoltaic 1", "Electric", "Software Sensor: SMRT", 100);
            SensorManager.getInstance().AddFilter("Photovoltaic 1", "Database", "Database Filter");
            SensorManager.getInstance().AddFilter("Photovoltaic 1", "Database Cleanup", "Database Cleanup Filter");
            SensorManager.getInstance().AddFilter("Photovoltaic 1", "Database Rollup", "Database Rollup Filter");
            SensorManager.getInstance().InstantiateSensor("Photovoltaic 2", "Electric", "Software Sensor: SMRT", 100);
            SensorManager.getInstance().AddFilter("Photovoltaic 2", "Database", "Database Filter");
            SensorManager.getInstance().AddFilter("Photovoltaic 2", "Database Cleanup", "Database Cleanup Filter");
            SensorManager.getInstance().AddFilter("Photovoltaic 2", "Database Rollup", "Database Rollup Filter");
            SensorManager.getInstance().InstantiateSensor("Turbine 1", "Wind", "Software Sensor: SMRT", 100);
            SensorManager.getInstance().AddFilter("Turbine 1", "Database", "Database Filter");
            SensorManager.getInstance().AddFilter("Turbine 1", "Database Cleanup", "Database Cleanup Filter");
            SensorManager.getInstance().AddFilter("Turbine 1", "Database Rollup", "Database Rollup Filter");
            SensorManager.getInstance().InstantiateSensor("Turbine 2", "Wind", "Software Sensor: SMRT", 100);
            SensorManager.getInstance().AddFilter("Turbine 2", "Database", "Database Filter");
            SensorManager.getInstance().AddFilter("Turbine 2", "Database Cleanup", "Database Cleanup Filter");
            SensorManager.getInstance().AddFilter("Turbine 2", "Database Rollup", "Database Rollup Filter");
            SensorManager.getInstance().StartAllSensors();
            Thread.Sleep(4000);
#endif
            WhoIAm = new Stack<KeyValuePair<string,string>>();
            WhoIAm.Push(new KeyValuePair<string, string>(Brewery.getInstance().UPRN, "Me"));
            PushNew(Brewery.getInstance().UPRN);
            levelsDown++;

            this.Show();
            InitializePanels();
            DrawSystemUI();
            cnt.Start();
            cnt2.Start();

            eve = new Thread(new ThreadStart(MessageLoop));
            eve.Name = "Task Event Message Loop";
            eve.Start();

            MSG message = default(MSG);
            while (running)
            {
                while (PeekMessage(ref message, 0, 0, 0, 1))
                {
                    TranslateMessage(ref message);
                    DispatchMessage(ref message);
                }
                CrankOut();
                Thread.Sleep(100);
            }
            Application.Exit();
            Brewery.getInstance().Dispose();
        }

        private void MessageLoop()
        {
            while (running)
            {
                CommunicationManager.getInstance().DoEvents();
            }
        }

        Stack<KeyValuePair<string, string>> WhoIAm;
        bool UserAuthorized;
        bool Connected;
        string userAttempt;
        string passAttempt;

        public void PushNew(string s)
        {
            UserAuthorized = false;
            Connected = false;
            userAttempt = null;
            passAttempt = null;
            WhoIAm.Push(new KeyValuePair<string, string>(s, "Me"));

        }

        private void CrankOut()
        {
            if (WhoIAm.Count == 0)
            {
                running = false;
            }
            else
            if (this.Enabled)
            {
                if (!Connected)
                {
                    if (ConnectionManager.getInstance().GetAllActiveConnections().Contains(WhoIAm.Peek().Key))
                    {
                        Connected = true;
                    }
                    else
                    {
                        while (WhoIAm.Count != 1)
                        {
                            WhoIAm.Pop();
                        }
                    }
                }
                else
                {
                    if (UserAuthorized)
                    {
                        StartSystemRefresh();
                        GetSensorData();
                    }
                    else
                    {
                        if (this.Enabled)
                        {
                            if (userAttempt == null)
                            {
                                StartPasswordDialog();
                            }
                            else
                            {
                                if (Brewery.getInstance().IsVariableSet(WhoIAm.Peek().Key + "_UserRecognition"))
                                {
                                    if ((bool)Brewery.getInstance().PopVariable(WhoIAm.Peek().Key + "_UserRecognition"))
                                    {
                                        UserAuthorized = true;
                                    }
                                    else
                                    {
                                        userAttempt = null;
                                    }
                                }
                                else
                                {
                                    Brewery.getInstance().AddVariable(WhoIAm.Peek().Key + "_UserRecognition");
                                    Packet p = new Packet(WhoIAm.Peek().Key, WhoIAm.Peek().Key + "_UserRecognition", MessageManager.GetSubjectOf(typeof(UserRecognitionRequestMessageHandler)), new UserInformation(userAttempt, passAttempt, true));
                                    SendData(WhoIAm.Peek().Key, p);
                                    StartLoadingDialog(WhoIAm.Peek().Key + "_UserRecognition");
                                }
                            }
                        }
                    }
                }
            }
            else
            {

            }
        }

        public void StartSystemRefresh()
        {
            lower = new List<string>();
            upper = new List<string>();
            SystemCounter--;
            if (cnt.Peek > SystemRefreshRate)
            {
                cnt.Stop();
                cnt.Start();


                if (WhoIAm.Count == 0) // Self!
                {
                    lower.AddRange(CommunicationManager.getInstance().LowerConnections);
                    upper.AddRange(CommunicationManager.getInstance().HigherConnections);
                }
                else // Below Self!
                {
                    string subject = MessageManager.GetSubjectOf(typeof(RegisteredSystemNumberRequestMessageHandler));
                    Packet p = new Packet(Brewery.getInstance().UPRN, WhoIAm.Peek().Key, subject, new StringData());

                    Brewery.getInstance().AddVariable(WhoIAm.Peek().Key + "_SystemList");

                    SendData(WhoIAm.Peek().Key, p);

                    while (!Brewery.getInstance().IsVariableSet(WhoIAm.Peek().Key + "_SystemList"))
                        Thread.Sleep(50);

                    object o = Brewery.getInstance().PopVariable(WhoIAm.Peek().Key + "_SystemList");
                    lower = (List<string>)o;
                }

                // Do something with tehSystems
                // lstHigherConnections.Items.Clear();
                // lstLowerConnections.Items.Clear();

                List<string> blar = new List<string>();
                for (int i = 0; i < lstHigherConnections.Items.Count; i++)
                {
                    blar.Add(lstHigherConnections.Items[i] as string);
                }


                foreach (string s in upper)
                {
                    if (blar.Contains(s))
                        blar.Remove(s);
                    else
                        if (!lstHigherConnections.Items.Contains(s))
                            lstHigherConnections.Items.Add(s);
                }
                foreach (string s in blar)
                {
                    lstHigherConnections.Items.Remove(s);
                }

                blar = new List<string>();
                for (int i = 0; i < lstLowerConnections.Items.Count; i++)
                {
                    blar.Add(lstLowerConnections.Items[i] as string);
                }
                foreach (string s in lower)
                {
                    if (blar.Contains(s))
                        blar.Remove(s);
                    else
                        if (!lstLowerConnections.Items.Contains(s))
                            lstLowerConnections.Items.Add(s);
                }
                foreach (string s in blar)
                {
                    lstLowerConnections.Items.Remove(s);
                }

                lstHigherConnections.Refresh();
                lstLowerConnections.Refresh();
                /*
                lstHigherConnections.Items.AddRange(uberSystems.ToArray());
                lstHigherConnections.Refresh();
                lstLowerConnections.Items.AddRange(tehSystems.ToArray());
                lstLowerConnections.Invalidate();
                */
                SystemCounter = SystemRefreshRate;
            }


        }

        private Packet EncapsePacket(Packet Request, int Level)
        {
            if (Level <= 0)
                return Request;
            Request = new Packet(Request.Sender, Request.Recipient, Request.Subject, new PacketData(Request));
            return EncapsePacket(Request, Level - 1);
        }

        private void SendData(string SendTo, Packet p)
        {
            SendDataTask t = new SendDataTask();
            t.InitializeTask(SendTo, new object[] { p, SecurityManager.getInstance().GetKey(SendTo, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(t);
        }

        private List<string> lower;
        private List<string> upper;


        private void GetSensorData()
        {
            if (cnt2.Peek > SensorRefreshRate)
            {
                cnt2.Stop();
                cnt2.Start();

                List<string> Sensors = new List<string>();

                if (WhoIAm.Count == 0)
                {
                    #region obsoleteish
                    Sensors.AddRange(SensorManager.getInstance().GetCreatedSensors());
                    foreach (string s in Sensors)
                    {
                        SensorsTableEntry ste = SensorsTableEntry.Find(SensorsTableEntry.NameColumn, s);
                        SnapshotsTableEntry e = SnapshotsTableEntry.Find(SnapshotsTableEntry.SensorColumn, ste.ID.ToString());

                        SensorReading r = new SensorReading();
                        r.Filters = new List<string>(SensorManager.getInstance().GetFiltersOnSensor(s).Keys);
                        if (e != null)
                        {
                            r.Avg = e.Average;
                            r.Max = e.Maximum;
                            r.Min = e.Minimum;
                            r.Total = e.Total;
                            r.Time = e.Timestamp;
                            r.Duration = e.Duration;
                        }
                        else
                        {
                            r.Avg = 0;
                            r.Max = 0;
                            r.Min = 0;
                            r.Total = 0;
                            r.Time = DateTime.Now.ToString();
                            r.Duration = 0;
                        }

                        if (!SensorData.ContainsKey(s))
                            SensorData.Add(s, new List<SensorReading>());
                        SensorData[s].Add(r);
                    }
                    #endregion
                }
                else
                {
                    string subject = MessageManager.GetSubjectOf(typeof(SensorListNumberRequestMessageHandler));
                    Packet p = new Packet(Brewery.getInstance().UPRN, WhoIAm.Peek().Key, subject, new StringData());

                    Brewery.getInstance().AddVariable(WhoIAm.Peek().Key + "_SensorList");

                    SendData(WhoIAm.Peek().Key, p);

                    while (!Brewery.getInstance().IsVariableSet(WhoIAm.Peek().Key + "_SensorList"))
                        Thread.Sleep(50);

                    object o = Brewery.getInstance().PopVariable(WhoIAm.Peek().Key + "_SensorList");
                    List<string> sense = (List<string>)o;
                    Sensors.AddRange(sense);

                    foreach (string s in sense)
                    {

                        SensorReading r = new SensorReading();

                        string sbj = MessageManager.GetSubjectOf(typeof(SensorDataRequestMessageHandler));
                        Packet pa = new Packet(Brewery.getInstance().UPRN, WhoIAm.Peek().Key, sbj, new StringData(s));

                        Brewery.getInstance().AddVariable(WhoIAm.Peek().Key + "_" + s + "_Data");

                        SendData(WhoIAm.Peek().Key, pa);

                        while (!Brewery.getInstance().IsVariableSet(WhoIAm.Peek().Key + "_" + s + "_Data"))
                            Thread.Sleep(50);

                        Dictionary<string, string> readings = Brewery.getInstance().PopVariable(WhoIAm.Peek().Key + "_" + s + "_Data") as Dictionary<string, string>;
                        // Name, Timestamp, Average, Min, Max, Total
                        r.Time = readings["Timestamp"];
                        r.Avg = double.Parse(readings["Avg"]);
                        r.Min = double.Parse(readings["Min"]);
                        r.Max = double.Parse(readings["Max"]);
                        r.Total = double.Parse(readings["Total"]);


                        if (!SensorData.ContainsKey(WhoIAm.Peek() + "_" + s))
                            SensorData.Add(WhoIAm.Peek() + "_" + s, new List<SensorReading>());
                        SensorData[WhoIAm.Peek() + "_" + s].Add(r);


                    }


                }
                RefreshSensorPanel();
            }       
        }
        

        private void RefreshSensorPanel()
        {
            foreach (string i in lstSensors.Items)
            {
                if (!SensorData.ContainsKey(i))
                    lstSensors.Items.Remove(i);
            }
            foreach (string s in SensorData.Keys)
            {
                if (!lstSensors.Items.Contains(s))
                    lstSensors.Items.Add(s);
            }

            lstSensors.Refresh();
            lstSensors_SelectedIndexChanged(lstSensors, new EventArgs());
        }

        private void GraphSensors()
        {

        }

        private void lstSensors_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstSensors.SelectedIndex < 0) return;
            string which = lstSensors.Items[lstSensors.SelectedIndex].ToString();
            int latest = SensorData[which].Count - 1;

            lstSensorsFilters.Items.Clear();
            lstSensorsFilters.Items.AddRange(SensorData[which][latest].Filters.ToArray());

            textBox1.Text = SensorData[which][latest].Total.ToString();
            textBox2.Text = SensorData[which][latest].Avg.ToString();
            textBox3.Text = SensorData[which][latest].Min.ToString();
            textBox4.Text = SensorData[which][latest].Max.ToString();
            textBox5.Text = SensorData[which][latest].Time.ToString();
            textBox6.Text = SensorData[which][latest].Duration.ToString();
        }

        private void InitializePanels()
        {
            NavPanelList = new Dictionary<string, Panel>();
            MainPanelList = new Dictionary<string, Panel>();
            ActivePanels = new List<Panel>();

            #region Navigation
            #region DonutPanel
            NavPanelList.Add("LogoNavigation", SplashPanel);
            SplashPanel.Visible = false;
            SplashPanel.Location = new Point(9000, 9000);
            SplashPanel.Width = splitContainer1.Panel1.Width - vScrollBar1.Width;
            SplashPanel.Height = SplashPanel.Width + pictureBox3.Height * 2;
            DonutLogo.Image = new Bitmap(Resources.TopLeft, new Size(SplashPanel.Width, SplashPanel.Width));
            #endregion

            #region BackPanel
            BackPanel.Visible = false;
            NavPanelList.Add("BackNavigation", BackPanel);
            BackPanel.Location = new Point(9000, 9000);
            BackPanel.Width = splitContainer1.Panel1.Width - vScrollBar1.Width;
            #endregion

            #region SearchPanel
            SearchPanel.Visible = false;
            NavPanelList.Add("SearchNavigation", SearchPanel);
            SearchPanel.Location = new Point(9000, 9000);
            SearchPanel.Width = splitContainer1.Panel1.Width - vScrollBar1.Width;
            #endregion

            #region SensorPanel
            SensorsPanel.Visible = false;
            NavPanelList.Add("SensorsNavigation", SensorsPanel);
            SensorsPanel.Location = new Point(9000, 9000);
            SensorsPanel.Width = splitContainer1.Panel1.Width - vScrollBar1.Width;
            #endregion

            #region GraphPanel
            GraphPanel.Visible = false;
            NavPanelList.Add("GraphNavigation", GraphPanel);
            GraphPanel.Location = new Point(9000, 9000);
            GraphPanel.Width = splitContainer1.Panel1.Width - vScrollBar1.Width;
            #endregion

            #region MapNavPanel
            MapNavPanel.Visible = false;
            NavPanelList.Add("MapNavigation", MapNavPanel);
            MapNavPanel.Location = new Point(9000, 9000);
            MapNavPanel.Width = splitContainer1.Panel1.Width - vScrollBar1.Width;
            #endregion

            #endregion

            #region Main

            #region MapPanel
            MapPanel.Visible = false;
            MainPanelList.Add("MapPanel", MapPanel);
            MapPanel.Location = new Point(9000, 9000);
            #endregion

            #region SensorPanel
            SensorMainPanel.Visible = false;
            MainPanelList.Add("SensorMainPanel", SensorMainPanel);
            SensorMainPanel.Location = new Point(9000, 9000);
            #endregion


            #endregion

            SetPanel(MainPanelList["SensorMainPanel"]);

            DrawSystemUI();
        }

        private void pain()
        {
            Form f = new SplashScreen(new Bitmap(Assembly.GetExecutingAssembly().GetManifestResourceStream("DONUT.splash.png")), this);
        }

        protected override void OnClosed(EventArgs e)
        {
            running = false;
            Application.Exit();
            DuffBeer.Brewery.getInstance().Dispose();
        }

        private void setVScrollHeight()
        {
            int porn = -splitContainer1.Panel1.Height;
            foreach (Panel p in ActivePanels)
            {
                porn += p.Height;
            }
            if (porn < 0)
                porn = 0;

            if (porn > 0)
            {
                vScrollBar1.Enabled = true;
                vScrollBar1.Minimum = 0;
                vScrollBar1.Maximum = porn;
            }
            else
            {
                vScrollBar1.Enabled = false;
            }
        }




        private void vScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            PositionSidebar(-e.NewValue);
        }

        private void ChangeSidebar()
        {
            setVScrollHeight();
            PositionSidebar(0);

        }

        private void PositionSidebar(int offset)
        {
            for (int i = 0; i < ActivePanels.Count; i++)
            {
                ActivePanels[i].Location = new Point(splitContainer1.Panel1.Left, offset);
                offset += ActivePanels[i].Height;
                ActivePanels[i].Visible = true;
            }
            this.Refresh();
        }

        private void SetPanel(Panel p)
        {
            foreach (KeyValuePair<string, Panel> k in MainPanelList)
            {
                k.Value.Dock = DockStyle.None;
                k.Value.Visible = false;
            }
            p.Dock = DockStyle.Fill;
            p.Visible = true;
        }

        private void StartPasswordDialog()
        {
            this.Enabled = false;
            Form l = new LoginDialog(this);
            Thread.Sleep(10);
            (l).ShowDialog();
        }

        public void CompletePasswordDialog(string user, string pass)
        {
            if (user == null || pass == null)
            {
                WhoIAm.Pop();
            }
            else
            {
                userAttempt = user;
                passAttempt = pass;
                
            }


            this.Enabled = true;
        }

        private void StartLoadingDialog(string variable)
        {

            this.Enabled = false;
            Form l = new LoadingDialog(this, variable);
            Thread.Sleep(10);
            (l).ShowDialog();
        }

        public void CompleteLoadingDialog()
        {
            this.Enabled = true;
        }

        private void StartGraphDialog(List<double> crap, DateTime time)
        {
            this.Enabled = false;
            Form l = new GraphDialog(this, crap, time);
            Thread.Sleep(10);
            (l).ShowDialog(this);
        }

        public void CompleteGraphDialog()
        {
            this.Enabled = true;
        }






        private void SensorsButton_Click(object sender, EventArgs e)
        {
            SetPanel(MainPanelList["SensorMainPanel"]);
            GetSensorData();
        }

        private void DrawSystemUI()
        {
            ActivePanels.Clear();
            if (WhoIAm.Count == 1)
                MainSystemUI();
            else
                LowerSystemUI();
        }

        private void MainSystemUI()
        {
            ActivePanels.Add(NavPanelList["LogoNavigation"]);
            ActivePanels.Add(NavPanelList["SensorsNavigation"]);
            ActivePanels.Add(NavPanelList["MapNavigation"]);
            ChangeSidebar();
        }

        private void LowerSystemUI()
        {
            ActivePanels.Add(NavPanelList["LogoNavigation"]);
            ActivePanels.Add(NavPanelList["BackNavigation"]);
            ActivePanels.Add(NavPanelList["SensorsNavigation"]);
            ActivePanels.Add(NavPanelList["MapNavigation"]);
            ChangeSidebar();
        }












        #region Windows Message Junk
        struct POINTAPI
        {
            internal Int32 x;
            internal Int32 y;
        }

        struct MSG
        {
            internal Int32 hwmd;
            internal Int32 message;
            internal Int32 wParam;
            internal Int32 lParam;
            internal Int32 time;
            internal POINTAPI pt;
        }

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool PeekMessage(ref MSG lpMsg, Int32 hwnd, Int32 wMsgFilterMin, Int32 wMsgFilterMax, Int32 wRemoveMsg);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool TranslateMessage(ref MSG lpMsg);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern Int32 DispatchMessage(ref MSG lpMsg);
        #endregion


        private void MapButton_Click(object sender, EventArgs e)
        {
            StartPasswordDialog();
        }

        private void SensorsDisplayGraph_Click(object sender, EventArgs e)
        {
            if (lstSensors.SelectedIndex < 0) return;
            string which = lstSensors.Items[lstSensors.SelectedIndex].ToString();

            List<double> data = new List<double>();
            foreach (SensorReading r in SensorData[which])
                data.Add(r.Avg);
            data.Sort();

            StartGraphDialog(data, DateTime.Now);
        }

        private void BackButton_Click_1(object sender, EventArgs e)
        {
            WhoIAm.Pop();
        }

        private void lstLowerConnections_SelectedIndexChanged(object sender, EventArgs e)
        {
            
        }

        private void lstLowerConnections_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                PushNew(lstLowerConnections.SelectedItem as string);
            }
            catch
            {
            }
        }

    }
}