using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Threading;
using System.Reflection;
using DuffBeer;
using MrHomer.Security;
using DuffBeer.Security.Cryptography;
using DuffBeerDriver.FakePlugins;
using DuffBeer.Security.Authentication;

namespace DONUT
{
    public class SplashScreen : Form
    {
        Bitmap splash;
        Bitmap other;
        Brewery b;

        public SplashScreen(Bitmap bmp, Form main)
        {
            Counter c = new Counter();
            c.Start();
            splash = bmp;
            this.TopMost = false;
            this.ShowInTaskbar = false;
            this.Visible = true;
            this.FormBorderStyle = FormBorderStyle.None;
            this.Size = new Size(bmp.Width, bmp.Height);
            this.Location = new Point(Screen.PrimaryScreen.WorkingArea.Width / 2 - this.Width / 2, Screen.PrimaryScreen.WorkingArea.Height / 2 - this.Height / 2);
            this.Show();
            Resources.Donut = (new Bitmap(Assembly.GetExecutingAssembly().GetManifestResourceStream("DONUT.donut.png")));
            Resources.TopLeft = (new Bitmap(Assembly.GetExecutingAssembly().GetManifestResourceStream("DONUT.topleft.jpg")));

            ThreadStart t = new ThreadStart(MakeBrew);
            Thread th = new Thread(t);
            th.Name = "Brewery Builder";
            th.Start();
            
            MSG message = default(MSG);

            while ((c.Peek < 5))
            {
                while (PeekMessage(ref message, 0, 0, 0, 1))
                {
                    TranslateMessage(ref message);
                    DispatchMessage(ref message);
                }
                CrankOut();
            }
            this.Dispose();
            return;
        }

        private void CrankOut()
        {
            other = new Bitmap(splash);
            Graphics g = Graphics.FromImage(other);
            g.DrawString("Loading DONUT...", new Font(FontFamily.GenericMonospace, 10), new SolidBrush(Color.White), new PointF(300, 200));
            SelectBitmap(other);
            this.Refresh();
        }

        private void MakeBrew()
        {
            SecurityManager.getInstance(new SuperAuth(), new NRotN_Crypto(), new SuperKey());
            b = Brewery.getInstance();
            return;
        }

        #region Bitmap Setting

        // Sets the current bitmap
        private void SelectBitmap(Bitmap bitmap)
        {
            // Does this bitmap contain an alpha channel?
            if (bitmap.PixelFormat != PixelFormat.Format32bppArgb)
            {
                throw new ApplicationException("The bitmap must be 32bpp with alpha-channel.");
            }

            // Get device contexts
            IntPtr screenDc = APIHelp.GetDC(IntPtr.Zero);
            IntPtr memDc = APIHelp.CreateCompatibleDC(screenDc);
            IntPtr hBitmap = IntPtr.Zero;
            IntPtr hOldBitmap = IntPtr.Zero;

            try
            {
                // Get handle to the new bitmap and select it into the current device context
                hBitmap = bitmap.GetHbitmap(Color.FromArgb(0));
                hOldBitmap = APIHelp.SelectObject(memDc, hBitmap);

                // Set parameters for layered window update
                APIHelp.Size newSize = new APIHelp.Size(this.Size.Width, this.Size.Height);
                APIHelp.Point sourceLocation = new APIHelp.Point(0, 0);
                APIHelp.Point newLocation = new APIHelp.Point(this.Left, this.Top);		// Same as this window
                APIHelp.BLENDFUNCTION blend = new APIHelp.BLENDFUNCTION();
                blend.BlendOp = APIHelp.AC_SRC_OVER;						// Only works with a 32bpp bitmap
                blend.BlendFlags = 0;											// Always 0
                blend.SourceConstantAlpha = 255;										// Set to 255 for per-pixel alpha values
                blend.AlphaFormat = APIHelp.AC_SRC_ALPHA;						// Only works when the bitmap contains an alpha channel

                // Update the window
                APIHelp.UpdateLayeredWindow(Handle, screenDc, ref newLocation, ref newSize,
                    memDc, ref sourceLocation, 0, ref blend, APIHelp.ULW_ALPHA);
            }
            finally
            {
                // Release device context
                APIHelp.ReleaseDC(IntPtr.Zero, screenDc);
                if (hBitmap != IntPtr.Zero)
                {
                    APIHelp.SelectObject(memDc, hOldBitmap);
                    APIHelp.DeleteObject(hBitmap);										// Remove bitmap resources
                }
                APIHelp.DeleteDC(memDc);
            }
        }

        protected override CreateParams CreateParams
        {
            get
            {
                // Add the layered extended style (WS_EX_LAYERED) to this window
                CreateParams createParams = base.CreateParams;
                createParams.ExStyle |= APIHelp.WS_EX_LAYERED;
                return createParams;
            }
        }

        class APIHelp
        {
            // Required constants
            internal const Int32 WS_EX_LAYERED = 0x80000;
            internal const Int32 HTCAPTION = 0x02;
            internal const Int32 WM_NCHITTEST = 0x84;
            internal const Int32 ULW_ALPHA = 0x02;
            internal const byte AC_SRC_OVER = 0x00;
            internal const byte AC_SRC_ALPHA = 0x01;

            internal enum Bool
            {
                False = 0,
                True = 1
            }

            [StructLayout(LayoutKind.Sequential)]
            internal struct Point
            {
                internal Int32 x;
                internal Int32 y;

                internal Point(Int32 x, Int32 y) { this.x = x; this.y = y; }
            }

            [StructLayout(LayoutKind.Sequential)]
            internal struct Size
            {
                internal Int32 cx;
                internal Int32 cy;

                internal Size(Int32 cx, Int32 cy) { this.cx = cx; this.cy = cy; }
            }

            [StructLayout(LayoutKind.Sequential, Pack = 1)]
            struct ARGB
            {
                internal byte Blue;
                internal byte Green;
                internal byte Red;
                internal byte Alpha;
            }

            [StructLayout(LayoutKind.Sequential, Pack = 1)]
            internal struct BLENDFUNCTION
            {
                internal byte BlendOp;
                internal byte BlendFlags;
                internal byte SourceConstantAlpha;
                internal byte AlphaFormat;
            }

            [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
            internal static extern Bool UpdateLayeredWindow(IntPtr hwnd, IntPtr hdcDst, ref Point pptDst, ref Size psize, IntPtr hdcSrc, ref Point pprSrc, Int32 crKey, ref BLENDFUNCTION pblend, Int32 dwFlags);

            [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
            internal static extern IntPtr CreateCompatibleDC(IntPtr hDC);

            [DllImport("user32.dll", ExactSpelling = true, SetLastError = true)]
            internal static extern IntPtr GetDC(IntPtr hWnd);

            [DllImport("user32.dll", ExactSpelling = true)]
            internal static extern int ReleaseDC(IntPtr hWnd, IntPtr hDC);

            [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
            internal static extern Bool DeleteDC(IntPtr hdc);

            [DllImport("gdi32.dll", ExactSpelling = true)]
            internal static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);

            [DllImport("gdi32.dll", ExactSpelling = true, SetLastError = true)]
            internal static extern Bool DeleteObject(IntPtr hObject);
        }


        #endregion

        #region Windows Message Junk
        struct POINTAPI
        {
            internal Int32 x;
            internal Int32 y;
        }

        struct MSG
        {
            internal Int32 hwmd;
            internal Int32 message;
            internal Int32 wParam;
            internal Int32 lParam;
            internal Int32 time;
            internal POINTAPI pt;
        }

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool PeekMessage(
                                       ref MSG lpMsg,
                                   Int32 hwnd,
                           Int32 wMsgFilterMin,
                           Int32 wMsgFilterMax,
                           Int32 wRemoveMsg);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool TranslateMessage(ref MSG lpMsg);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern Int32 DispatchMessage(ref MSG lpMsg);
        #endregion
    

    }
}
