using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Networking;

namespace DuffBeer.Communication
{

    public enum AuthenticationType
    {
        Connection,
        User
    };

    public class CommunicationManager
    {
        #region Singleton

        private static CommunicationManager instance;

        public static CommunicationManager getInstance()
        {
            if (instance == null)
                instance = new CommunicationManager();
            return instance;
        }

        #endregion

        public volatile List<string> LowerConnections;
        public volatile List<string> HigherConnections;
        public volatile List<string> TempLowerConnections;
        public volatile List<string> TempHigherConnections;

        public volatile List<string> PendingHigherConnection;
        public volatile List<string> PendingTempHigherConnection;
        public volatile List<string> PendingLowerConnection;
        public volatile List<string> PendingTempLowerConnection;

        public List<string> Disconnected;

        public List<ITask> TasksToAdd;
        
        private bool ShuttingDown;
        private Queue<ITask> tasks;

        private CommunicationManager()
        {
            tasks = new Queue<ITask>();
            TasksToAdd = new List<ITask>();
            LowerConnections = new List<string>();
            HigherConnections = new List<string>();
            TempLowerConnections = new List<string>();
            TempHigherConnections = new List<string>();

            PendingHigherConnection = new List<string>();
            PendingTempHigherConnection = new List<string>();
            PendingLowerConnection = new List<string>();
            PendingTempLowerConnection = new List<string>();


            ShuttingDown = false;
        }



        public bool HasEvents
        {
            get { return (tasks.Count + TasksToAdd.Count) > 0; }
        }

        public void DoEvents()
        {
            while(tasks.Count > 0)
            {
                ITask t;

                lock (tasks)
                {
                    t = tasks.Dequeue();
                }

                t.PerformTask();
            }
            if (!ShuttingDown)
            {
                foreach (ITask lol in TasksToAdd)
                {
                    tasks.Enqueue(lol);
                }
            }
            TasksToAdd.Clear();
        }

        public void AddTask(ITask task)
        {
            lock (TasksToAdd)
            {
                TasksToAdd.Add(task);
            }
        }

        public void ShutDown()
        {
            ShuttingDown = true;
            ConnectionManager.getInstance().Dispose();
        }
    }
}
