using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Networking;
using MrHomer.Security;
using DuffBeer.Database.StructureTypes;
using MrHomer;

namespace DuffBeer.Communication
{
    public class ActivationTask : ITask
    {
        int port;
        CommunicationManager cmm;
        public ActivationTask(int port)
        {
            this.port = port;
            cmm = CommunicationManager.getInstance();
            ConnectionManager.getInstance().ConnectionActivated += new ConnectionManager.Connection(ActivateConnection);
        }

        public void PerformTask()
        {
            ConnectionManager cm = ConnectionManager.getInstance();
            SecurityManager sm = SecurityManager.getInstance();
            foreach (KeyValuePair<string, byte[]> inactive in cm.GetAllInactiveConnections())
            {
                try
                {
                    Packet p = new Packet(inactive.Value);
                    Credentials c = p.Body as Credentials;
                    if (sm.Authenticate(c.UPRN, AuthenticationType.Connection.ToString(), new object[] { }))
                    {
                        cm.ActivateConnection(inactive.Key, c.UPRN);

                        SystemsTableEntry table = SystemsTableEntry.Find(DuffBeerDatabaseConfig.RegisteredSystemsTable, SystemsTableEntry.HostIDColumn, c.UPRN);
                        if (table == null)
                        {
                            CommunicationManager.getInstance().TempLowerConnections.Add(c.UPRN);
                        }
                        else
                        {
                            CommunicationManager.getInstance().LowerConnections.Add(c.UPRN);
                        }
                    }
                    else
                    {
                        cm.KillInactiveConnection(inactive.Key);
                    }

                }
                catch (Exception e)
                {
                    ErrorManager.getInstance().Catch(e);
                }
            }

            List<string> Dropped = cm.GetAllActiveConnections();
            StringDiff(cmm.HigherConnections, Dropped);
            StringDiff(cmm.TempHigherConnections, Dropped);
            Dropped = cm.GetDroppedConnections(true);
            OtherStringDiff(cmm.HigherConnections, Dropped);
            OtherStringDiff(cmm.TempHigherConnections, Dropped);
            OtherStringDiff(cmm.TempLowerConnections, Dropped);
            OtherStringDiff(cmm.LowerConnections, Dropped);
            Dropped = cm.GetFailedConnections(true);
            OtherStringDiff(cmm.HigherConnections, Dropped);
            OtherStringDiff(cmm.TempHigherConnections, Dropped);
            OtherStringDiff(cmm.TempLowerConnections, Dropped);
            OtherStringDiff(cmm.LowerConnections, Dropped);
            
            Database.StructureTypes.ConnectionInitiationTableEntry mtable = new DuffBeer.Database.StructureTypes.ConnectionInitiationTableEntry();
            if (mtable != null && mtable.RowCount > 0)
            {
                do
                {
                    if (!CommunicationManager.getInstance().HigherConnections.Contains(mtable.HostUPRN))
                    {
                        ConnectionTask task = new ConnectionTask();
                        task.InitializeTask(mtable.HostUPRN, new object[] { mtable.HostAddress, port, true });
                        CommunicationManager.getInstance().AddTask(task);
                    }

                } while (mtable.Next());
            }


            CommunicationManager.getInstance().AddTask(this);
        }

        public void InitializeTask(string name, object[] data)
        {
        }

        private void ActivateConnection(string s)
        {
            if (cmm.PendingHigherConnection.Contains(s))
            {
                cmm.PendingHigherConnection.Remove(s);
                cmm.HigherConnections.Add(s);
            } if (cmm.PendingTempHigherConnection.Contains(s))
            {
                cmm.TempHigherConnections.Add(s);
            } if (cmm.PendingLowerConnection.Contains(s))
            {
                cmm.PendingLowerConnection.Remove(s);
                cmm.LowerConnections.Add(s);
            } if (cmm.PendingTempLowerConnection.Contains(s))
            {
                cmm.PendingTempLowerConnection.Remove(s);
                cmm.TempLowerConnections.Add(s);
            }
        }

        private void StringDiff(List<string> removeme, List<string> Dictionary)
        {
            List<string> log = new List<string>();
            foreach (string s in removeme)
            {
                if (!Dictionary.Contains(s))
                {
                    log.Add(s);
                }
            }
            foreach (string s in log)
                removeme.Remove(s);

        }


        private void OtherStringDiff(List<string> removeme, List<string> Dictionary)
        {
            List<string> log = new List<string>();
            foreach (string s in removeme)
            {
                if (Dictionary.Contains(s))
                {
                    log.Add(s);
                }
            }
            foreach (string s in log)
                removeme.Remove(s);

        }
    }
}
