using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using MrHomer.Database;
using MrHomer.Plugins;

namespace DuffBeer.Database
{
    [MrHomerPlugin("Type", "MSSQL_SQLAuth")]
    public class SQL_Database : IDatabase
    {
        protected SqlConnection conn;

        /// <summary>
        /// Creates a new MS SQL database
        /// </summary>
        public SQL_Database() { }

        #region IDatabase Members

        /// <summary>
        /// Creates a connection to the database
        /// </summary>
        /// <param name="dataSource">the DSN to use (usually a host name and port)</param>
        /// <param name="username">The username</param>
        /// <param name="password">The password</param>
        /// <param name="database">The database to use</param>
        public virtual bool Connect(string dataSource, string username, string password, string database)
        {
            SqlConnectionStringBuilder n = new SqlConnectionStringBuilder();

            n.DataSource = dataSource;
            n.IntegratedSecurity = false;
            n.Password = password;
            n.UserID = username;
            n.InitialCatalog = database;

            try
            {
                conn = new SqlConnection(n.ConnectionString);
                conn.Open();
            }
            catch
            {
                conn = null;
                return false;
            }

            return true;
        }

        /// <summary>
        /// Disconnects from the database
        /// </summary>
        public void Disconnect()
        {
            conn.Close();
        }

        /// <summary>
        /// Runs a query on this database
        /// </summary>
        /// <param name="queryString">The query</param>
        public DBResults Query(string queryString)
        {
            lock (conn)
            {
                SqlCommand n = new SqlCommand(queryString, conn);
                SqlDataReader r = n.ExecuteReader();
                DBResults res = null;

                try
                {
                    string[] keys = new string[r.VisibleFieldCount];
                    List<string[]> data = new List<string[]>();

                    for (int i = 0; i < r.VisibleFieldCount; i++)
                    {
                        keys[i] = r.GetName(i);
                    }

                    //do
                    while (r.Read())
                    {
                        object[] tmp = new object[r.VisibleFieldCount];
                        string[] tmp2 = new string[r.VisibleFieldCount];

                        r.GetSqlValues(tmp);

                        for (int k = 0; k < tmp.Length; k++)
                            tmp2[k] = tmp[k].ToString();

                        data.Add(tmp2);
                    } //while (r.Read());

                    res = new DBResults(keys, data.ToArray());
                }
                catch
                {
                    // Do nothing?
                }
                finally
                {
                    r.Close();
                }

                return res;
            }
        }

        /// <summary>
        /// Checks if this database is connected
        /// </summary>
        public bool IsConnected()
        {
            return conn != null;
        }

        #endregion

        #region IPlugin Members

        public string Author
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public string Version
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public string Label
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        #endregion


    }
}
