using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Database;

namespace DuffBeer.Database.StructureTypes
{
    public class KeyStoreTableEntry : TableEntry
    {
        /// <summary>
        /// Gets the text Name column name
        /// </summary>
        static public string NameColumn
        {
            get { return "Name"; }
        }
        /// <summary>
        /// Gets the text AuthKey column name
        /// </summary>
        static public string AuthKeyColumn
        {
            get { return "AuthKey"; }
        }

        /// <summary>
        /// Gets the key name from the current row
        /// </summary>
        public string Name
        {
            get { return myTableRef.Row[NameColumn]; }
            set { myTableRef.Row[NameColumn] = value; }
        }

        /// <summary>
        /// Gets the authentication key from the current row
        /// </summary>
        public string AuthKey
        {
            get { return myTableRef.Row[AuthKeyColumn]; }
            set { myTableRef.Row[AuthKeyColumn] = value; }
        }

        /// <summary>
        /// Gets first row from the KeyStore table
        /// </summary>
        public KeyStoreTableEntry()
            : base(DuffBeerDatabaseConfig.KeyStoreTable)
        {
        }

        /// <summary>
        /// Gets the specified row from the table
        /// </summary>
        /// <param name="RowID">The row ID</param>
        public KeyStoreTableEntry(int RowID)
            : this()
        {
            myTableRef.Find("ID", RowID.ToString());
        }

        /// <summary>
        /// Adds a new row to the KeyStore table
        /// </summary>
        /// <param name="Name">The new key's Name</param>
        /// <param name="AuthKey">The new key's Key</param>
        public KeyStoreTableEntry(string Name, string AuthKey)
            : this()
        {
            myTableRef.AddRow();
            this.Name = Name;
            this.AuthKey = AuthKey;
        }

        /// <summary>
        /// Finds the first row matching the condition
        /// </summary>
        /// <param name="Column">The column to search</param>
        /// <param name="Value">The value to find</param>
        public static KeyStoreTableEntry Find(string Column, string Value)
        {
            DataTable t = new DataTable(DatabaseManager.getInstance().FindDatabase(DuffBeerDatabaseConfig.DatabaseName), DatabaseManager.getInstance().GetQueryBuilder(DuffBeerDatabaseConfig.DatabaseName), DuffBeerDatabaseConfig.KeyStoreTable, "ID");
            if (t.Find(Column, Value))
                return new KeyStoreTableEntry(int.Parse(t.Row["ID"]));
            else
                return null;
        }
    }
}
