using System;
using System.IO;
using System.Text;
using System.Reflection;
using System.Collections.Generic;
using System.Threading;
using MrHomer.Database;
using MrHomer.Database.Query;
using MrHomer.Networking;
using MrHomer.Plugins;
using MrHomer.Security;
using DuffBeer.Communication;
using DuffBeer.Database;
using DuffBeer.Security.Authentication;
using DuffBeer.Security.Cryptography;
using DuffBeer.Security.Key_Stores;
using DuffBeer.Sensors;
using DuffBeer.Messages;
using System.Text.RegularExpressions;
using DuffBeer.Database.StructureTypes;
using MrHomer;

namespace DuffBeer
{
    public class Brewery : IDisposable
    {
        private string MyName;
        private string MyUPRN;

        private string state;
        public string GetState
        {
            get
            {
                if (state == null)
                    state = "";
                return state;
            }

        }

        private bool load;
        public bool Loading
        {
            get { return load; }
        }

        public string Name
        {
            get { return MyName; }
        }
        public string UPRN
        {
            get { return MyUPRN; }
        }

        Dictionary<string, object> Variables;

        public bool AddVariable(string s)
        {
            lock (Variables)
            {
                if (Variables.ContainsKey(s))
                    return false;
                Variables.Add(s, null);
                return true;
            }
        }

        public object PopVariable(string s)
        {
            lock (Variables)
            {
                if (Variables.ContainsKey(s))
                {
                    object a = Variables[s];
                    Variables.Remove(s);
                    return a;
                }
                return null;
            }
        }

        public bool IsVariableSet(string s)
        {
            lock (Variables)
            {
                if (Variables.ContainsKey(s))
                {
                    if (Variables[s] != null)
                    {
                        return true;
                    }
                }
                return false;
            }
        }

        public bool SetVariable(string s, object o)
        {
            lock (Variables)
            {
                if (Variables.ContainsKey(s))
                {
                    Variables[s] = o;
                    return true;
                }
                return false;
            }
        }

        public object PeekVariable(string s)
        {
            lock (Variables)
            {
                if (Variables.ContainsKey(s))
                    return Variables[s];
                return null;
            }
        }

        static private Brewery instance;
        static public Brewery getInstance()
        {
            if (instance == null)
                instance = new Brewery(0);
            return instance;
        }

        private Dictionary<string, string> Configuration;

        public string Config(string key)
        {
            if (Configuration.ContainsKey(key))
                return Configuration[key];
            return default(string);
        }

        private Brewery(int sleep)
            : this("recipe.ini", sleep)
        {
        }

        public Brewery(string configFile, int sleep)
        {
            try
            {
                state = "";
                load = true;

                {
                    state = "Beginning Loading...";
                }
                Thread.Sleep(sleep);
                #region MrHomer

                {
                    state = "Loading Configuration...";
                }
                Thread.Sleep(sleep);
                // LOAD CONFIGURATION
                LoadDefaultConfig();
                ParseConfigFile(configFile);

                this.MyUPRN = Configuration["UPRN"];
                this.MyName = Configuration["BreweryName"];
                this.Variables = new Dictionary<string, object>();

                // INITIALIZE PLUGIN MANAGER

                {
                    state = "Initializing Plugins...";
                }
                Thread.Sleep(sleep);
                PluginManager.getInstance().LoadInterfaceTypes(Assembly.GetExecutingAssembly());
                DatabaseManager.getInstance();
                PluginManager.getInstance().LoadPluginTypes(Assembly.GetExecutingAssembly());
                PluginManager.getInstance().LoadPluginTypes(Assembly.GetEntryAssembly());
                string pluginFolder = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "\\" + Configuration["PluginFolder"] + "\\";
                if (!Directory.Exists(pluginFolder))
                    Directory.CreateDirectory(pluginFolder);

                string[] plugins = Directory.GetFiles(pluginFolder, "*.dll", SearchOption.TopDirectoryOnly);
                foreach (string p in plugins)
                {

                    {
                        state = "Loading " + p;
                    }
                    PluginManager.getInstance().LoadPluginTypes(p);
                }

                {
                    state = "Initializing Database...";
                }
                Thread.Sleep(sleep);

                // INITIALIZE DATABASE MANAGER
                DatabaseManager.getInstance().RegisterTypes();
                DuffBeerDatabaseConfig.DatabaseName = Configuration["PrimaryDBName"];
                DatabaseManager.getInstance().ConnectDatabase(DuffBeerDatabaseConfig.DatabaseName, Configuration["PrimaryDBType"], Configuration["PrimaryDBURI"], Configuration["PrimaryDBUser"], Configuration["PrimaryDBPassword"], Configuration["PrimaryDBName"], null);


                {
                    state = "Initializing Security...";
                }
                Thread.Sleep(sleep);
                // INITIALIZE SECURITY MANAGER
                SecurityManager.getInstance(new DatabaseAuth(), new NRotN_Crypto(), new DBKeyStore());



                {
                    state = "Initializing Networking...";
                }
                Thread.Sleep(sleep);
                // INITIALIZE CONNECTION MANAGER
                ConnectionManager.getInstance().SetDropout(10000, true);

                Credentials cred = new Credentials();
                cred.UPRN = Configuration["UPRN"];

                Packet credentials = new Packet(Configuration["BreweryName"], Configuration["DefaultRecipient"], BrewerySettings.CredentialSubject, cred);
                ConnectionManager.getInstance().SetCredentials(credentials.Serialize());

                Packet request = new Packet(Configuration["DefaultRecipient"], Configuration["DefaultRecipient"], BrewerySettings.RequestSubject, new StringData(Configuration["RequestPacket"]));
                ConnectionManager.getInstance().SetRequestPacket(request.Serialize());

                #endregion

                #region DUFFBEER

                {
                    state = "Initializing Sensor Management...";
                }
                Thread.Sleep(sleep);

                SensorManager.getInstance().LoadSensorsAndFilters();
                InitializationTask task = new InitializationTask();
                task.InitializeTask(null, new object[] { int.Parse(Configuration["Port"]) });
                CommunicationManager.getInstance().AddTask(task);

                #endregion

                instance = this;
                load = false;
            }
            catch
            { }

        }

        private void LoadDefaultConfig()
        {
            Configuration = new Dictionary<string, string>();

            Configuration.Add("BreweryName", "UNCONFIGURED");
            Configuration.Add("UPRN", "UPRN013958710");
            Configuration.Add("DefaultRecipient", "KITTEH");
            Configuration.Add("Credentials", "HAS U SEEN MAH BUKKIT?");
            Configuration.Add("CredentialsKey", "OMGWTFBBQ");
            Configuration.Add("RequestPacket", "I CAN HAS CHEEZBURGER?");
            Configuration.Add("RequestKey", "QBBFTWGMO");
            Configuration.Add("Port", "2101");
            Configuration.Add("ServerPort", "2102");
            Configuration.Add("PluginFolder", "plugin");

            Configuration.Add("PrimaryDBName", "MrHomer");
            Configuration.Add("PrimaryDBType", "MSSQL_NTAuth");
            Configuration.Add("PrimaryDBURI", @".\sqlexpress");
            Configuration.Add("PrimaryDBUser", "");
            Configuration.Add("PrimaryDBPassword", "");
            Configuration.Add("LogDelay", "300000"); 
            //Configuration.Add("PVSensor1_Address", "");
            //Configuration.Add("PVSensor1_Port", "");
            //Configuration.Add("PVSensor1_Username", "");
            //Configuration.Add("PVSensor1_Password", "");           
        }

        private void ParseConfigFile(string filename)
        {
            string[] lines = new string[] { };
            Regex r = null;
            try
            {
                lines = File.ReadAllLines(filename);
                r = new Regex("(\\w.*?)\\s*=\\s*([\"']?)(.+)(\\2)", RegexOptions.IgnoreCase);
            }
            catch (Exception e)
            {
                ErrorManager.getInstance().Catch(e);
                return;
            }
            if (lines.Length == 0) return;
            if (lines[0] != "[Configuration]") return;

            for (int i = 1; i < lines.Length; i++)
            {
                Match m = r.Match(lines[i]);
                if (Configuration.ContainsKey(m.Groups[1].Value))
                    Configuration[m.Groups[1].Value] = m.Groups[3].Value;
                /*
                 * added by nroy to have properties that are in the config file that are not expected
                 * (I.E. properties that have not already been added to the dictionary) be added to the
                 * dictionary dynamically.  This allows us to create properties through the config file
                 * without hard coding them into the brewery.
                 */
                else
                    Configuration.Add(m.Groups[1].Value, m.Groups[3].Value);    
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            SensorManager.getInstance().StopAllSensors();
            CommunicationManager.getInstance().ShutDown();
            instance = null;
        }

        #endregion
    }
}
