using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Networking;
using MrHomer.Plugins;

namespace DuffBeer.Messages
{
    public class MessageManager
    {
        static private MessageManager instance;
        static public MessageManager getInstance()
        {
            if (instance == null)
                instance = new MessageManager();
            return instance;
        }

        private Dictionary<string, IMessageHandler> handlers;
        private IMessageHandler defMan;
        private PluginManager pm;

        private MessageManager()
        {
            handlers = new Dictionary<string, IMessageHandler>();
            pm = PluginManager.getInstance();
            List<Type> types = pm.getPlugins(typeof(IMessageHandler));
            List<Type> remove = new List<Type>();
            foreach (Type t in types)
            {
                if (!pm.EnforceAttributes("Required", t))
                {
                    remove.Add(t);
                }
            }
            foreach (Type t in remove)
            {
                types.Remove(t);
            }
            foreach (Type t in types)
            {
                foreach (MrHomerPluginAttribute att in pm.getAttributes(t))
                {
                    if (att.Name == "Subject")
                    {
                        handlers.Add(att.Value, pm.CreateInstance<IMessageHandler>(t,null));
                    }
                }
            }
        }

        public List<string> ListHandlers()
        {
            lock (handlers)
                return new List<string>(handlers.Keys);
        }

        public bool SetDefault(string s)
        {
            lock (handlers)
            {
                if (handlers.ContainsKey(s))
                {
                    defMan = handlers[s];
                    return true;
                }
            }
            return false;
        }

        public void Handle(Packet p, string Connection)
        {
            lock (handlers)
            {
                if (p.Subject != null && handlers.ContainsKey(p.Subject))
                    handlers[p.Subject].Invoke(p, Connection);
                else
                    if (defMan != null)
                        defMan.Invoke(p, Connection);
            }
        }

        public static string GetSubjectOf(Type t)
        {
            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(t))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            return subject;
        }
    }
}
