using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Messages;
using DuffBeer.Messages.ProgressTaskStructures;
using DuffBeer.Communication;
using DuffBeer.Main;
using MrHomer.Plugins;
using MrHomer.Networking;
using MrHomer.Security;


namespace DuffBeer.Messages.MessageTypes
{
    [MrHomerPlugin("Subject", "Locate by Label")]
    public class LocateLabelMessageHandler : IMessageHandler
    {
        #region IMessageHandler Members

        //  Packet
        // Sender       =   Irrelevent
        // Recipient    =   Put Back In
        // Body         =   SearchString, SearchString

        public void Invoke(MrHomer.Networking.Packet p, string Connection)
        {
            StringData w = p.Body as StringData;
            string search = SearchResult.Sanitize(w.Data);
            Dictionary<String, LocationTask> dict = LocationTaskList.getInstance().GetTasks();
            if (!dict.ContainsKey(search))
            {
                dict.Add(search, new LocationTask());
            }
            dict[search].SendTo = Connection;
            CommunicationManager cms = CommunicationManager.getInstance();
            SecurityManager sm = SecurityManager.getInstance();
            foreach (string s in cms.LowerConnections)
            {
                SendDataTask task = new SendDataTask();
                task.InitializeTask(s, new object[] {new Packet(p.Sender, p.Recipient, p.Subject, p.Body),sm.GetKey(s,null),null });
                cms.AddTask(task);
            }
            
            string UPRN = Brewery.getInstance().Config("UPRN");
            string Name = Brewery.getInstance().Config("BreweryName");
            if (UPRN.Contains(search) || Name.Contains(search))
            {
                SendDataTask task = new SendDataTask();

                string subject = "";
                foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(LocationReportLabelMessageHandler)))
                {
                    if (n.Name == "Subject")
                        subject = n.Value;
                }
                
                task.InitializeTask(Connection, new object[] {new Packet(p.Sender, p.Recipient, subject, p.Body), sm.GetKey(Connection, null), null });
                cms.AddTask(task);
            }
        }

        #endregion

        #region IPlugin Members

        public string Author
        {
            get { return "Chris Scalabrini"; }
        }

        public string Version
        {
            get { return "1.0"; }
        }

        public string Label
        {
            get { return "Sends back sensor data."; }
        }

        #endregion
    }
}
