using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Plugins;
using MrHomer.Networking;
using MrHomer.Security;
using DuffBeer.Sensors;

namespace DuffBeer.Messages.MessageTypes
{
    [MrHomerPlugin("Subject", "Requesting Real-Time Reporting")]
    public class RealTimeReportingRequestMessageHandler : IMessageHandler
    {
        #region IMessageHandler Members
        
        public void Invoke(MrHomer.Networking.Packet p, string Connection)
        {
            StringData sensor = p.Body as StringData;
            List<string> sensors = SensorManager.getInstance().GetCreatedSensors();
            //if the sensor is already instantiated
            if (sensors.Contains(sensor.ToString()))
            {
                //if the sensor doesn't have a real-time data filter on it
                if (!SensorManager.getInstance().GetFiltersOnSensor(sensor.ToString()).ContainsKey("Real-Time Data Filter"))
                {
                    //add a real time data filter to the sensor
                    SensorManager.getInstance().AddFilter(sensor.ToString(), "Real-Time Data Filter", "Real Time Reporting Filter");
                }

            }
            else return;        //can't get real time reports from a non-existent sensor
        }

        #endregion

        #region IPlugin Members

        public string Author
        {
            get { return "Nathan Roy"; }
        }

        public string Version
        {
            get { return "V 0.1"; }
        }

        public string Label
        {
            get { return "Adds a real-time data filter to a sensor for all connection"; }
        }

        #endregion
    }
}
