using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Messages;
using MrHomer.Plugins;
using DuffBeer.Messages.ProgressTaskStructures;
using MrHomer.Networking;
using DuffBeer.Communication;

namespace DuffBeer.Messages.MessageTypes
{
    [MrHomerPlugin("Subject", "Registered Machine Count")]
    public class RegisteredSystemNumberResponseMessageHandler : IMessageHandler
    {
        #region IMessageHandler Members

        public void Invoke(MrHomer.Networking.Packet p, string Connection)
        {
            SubSystemTaskList lst = SubSystemTaskList.getInstance();
            int num = int.Parse((p.Body as StringData).ToString());

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(RegisteredSystemNumberResponseMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            if (!lst.GetTasks().ContainsKey(Connection))
            {
                lst.GetTasks().Add(Connection, new SubSystemTask(num));
                Reply(Connection, num, lst);
            }
            else if (lst.GetTasks()[Connection].Done)
            {
                lst.GetTasks()[Connection] = new SubSystemTask(num);
                Reply(Connection, num, lst);
            }
        }

        private void Reply(string Connection, int num, SubSystemTaskList lst)
        {
            if (num == 0)
            {
                Brewery.getInstance().SetVariable(Connection + "_SystemList", lst.GetTasks()[Connection].UPRNs);
                return;
            }

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(RegisteredSystemRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Packet p = new Packet(Brewery.getInstance().UPRN, Connection, subject, new StringData(""));
            SendDataTask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, MrHomer.Security.SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);
        }

        #endregion

        #region IPlugin Members

        public string Author
        {
            get { return "Brian O'Keefe"; }
        }

        public string Version
        {
            get { return "1.0"; }
        }

        public string Label
        {
            get { return "Sends back the number of registered systems."; }
        }

        #endregion
    }
}
