using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Messages;
using MrHomer.Plugins;
using DuffBeer.Database.StructureTypes;
using MrHomer.Networking;
using DuffBeer.Communication;

namespace DuffBeer.Messages.MessageTypes
{
    [MrHomerPlugin("Subject", "Requesting Sensor Data")]
    public class SensorDataRequestMessageHandler : IMessageHandler
    {
        #region IMessageHandler Members
        
        //Delegate and event for notifying the console that a sensor data request occured
        public delegate void SensorDataRequestNotification(string data);
        public static SensorDataRequestNotification RequestNotification;

        public void Invoke(MrHomer.Networking.Packet p, string Connection)
        {
            StringData d = p.Body as StringData;

            SensorsTableEntry s = SensorsTableEntry.Find(SensorsTableEntry.NameColumn, d.ToString());
            SnapshotsTableEntry e = SnapshotsTableEntry.Find(SnapshotsTableEntry.SensorColumn, s.ID.ToString());

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(SensorDataResponseMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }
            
            StringListData l = new StringListData();
            if (e != null)
            {
                l.Add(d.Data);
                l.Add(e.Timestamp.ToString());
                l.Add(e.Average.ToString());
            }
            else            //if the data couldn't be found in the database, just return NOT FOUND
            {
                l.Add(d.Data);
                l.Add(DateTime.Now.ToString());
                l.Add("0");
            }
            //l.Add(e.Minimum.ToString());
            //l.Add(e.Maximum.ToString());
            //l.Add(e.Total.ToString());
                        
            Packet z = new Packet(Brewery.getInstance().UPRN, p.Sender, subject, l);
            SendDataTask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { z, MrHomer.Security.SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            //Added by nroy to notify the console of the request
            if(RequestNotification != null)
                RequestNotification("Computer at: " + Connection + " requested data from " + d.ToString());
        }

        #endregion

        #region IPlugin Members

        public string Author
        {
            get { return "Chris Scalabrini (Modified by Nathan Roy)"; }
        }

        public string Version
        {
            get { return "1.0"; }
        }

        public string Label
        {
            get { return "Sends back sensor data."; }
        }

        #endregion
    }
}
