using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Security;
using MrHomer.Database;
using MrHomer.Database.Query;
using DuffBeer;
using DuffBeer.Database.StructureTypes;

namespace DuffBeer.Security.Authentication
{
    public class DatabaseAuth : IAuthenticator
    {
        /// <summary>
        /// Compares two arrays for equality
        /// </summary>
        /// <param name="a">The first array</param>
        /// <param name="b">The second array</param>
        /// <returns>True if the arrays are equivalent</returns>
        private bool ArrayEquals(Array a, Array b)
        {
            if (a.Length != b.Length) return false;

            try
            {
                for (int i = 0; i < a.Length; i++)
                    if (!a.GetValue(i).Equals(b.GetValue(i)))
                        return false;
            }
            catch
            {
                return false;
            }

            return true;
        }

        #region IAuthenticator Members

        /// <summary>
        /// Attempts to determine if the specified object is registered
        /// </summary>
        /// <param name="Name">The object to check</param>
        /// <param name="Type">The type of object</param>
        /// <param name="Details">Additional information about the object</param>
        /// <returns>The object's registration status</returns>
        public bool IsRegistered(string Name, string Type, object[] Details)
        {
            if (Type == "User")
                return RegisteredUsersTableEntry.Find(RegisteredUsersTableEntry.UsernameColumn, Name) != null;
            else
                return SystemsTableEntry.Find(DuffBeerDatabaseConfig.RegisteredSystemsTable, SystemsTableEntry.HostIDColumn, Name) != null
                    || SystemsTableEntry.Find(DuffBeerDatabaseConfig.TemporarySystemsTable, SystemsTableEntry.HostIDColumn, Name) != null;
        }

        /// <summary>
        /// Authenticates an object
        /// </summary>
        /// <param name="Name">The object to authenticate</param>
        /// <param name="Type">The type of object</param>
        /// <param name="Details">Additional information about the object</param>
        /// <returns>The object's authentication status</returns>
        public bool Authenticate(string Name, string Type, object[] Details)
        {
            if (Type == "User")
            {
                RegisteredUsersTableEntry r = RegisteredUsersTableEntry.Find(RegisteredUsersTableEntry.UsernameColumn, Name);
                if (r == null) return false;
                
                object key = r.Password;

                byte[] k = Encoding.UTF8.GetBytes(key.ToString());
                byte[] j = (Details.Length > 0 && Details[0] != null) ? (byte[])Details[0] : new byte[] { };

                return ArrayEquals(k, j);
            }
            else
            {
                if (SystemsTableEntry.Find(DuffBeerDatabaseConfig.RegisteredSystemsTable, SystemsTableEntry.HostIDColumn, Name) != null)
                    return true;
                else if (SystemsTableEntry.Find(DuffBeerDatabaseConfig.TemporarySystemsTable, SystemsTableEntry.HostIDColumn, Name) != null)
                    return true;
                else
                    return false;
            }
        }

        #endregion
    }
}
