using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Security;

namespace DuffBeer.Security.Authentication
{
    public sealed class Multi_Authenticator : List<IAuthenticator>, IAuthenticator
    {
        #region IAuthenticator Members

        /// <summary>
        /// Determines if any of the Authenticators have the named object registered
        /// </summary>
        /// <param name="Name">The object name</param>
        /// <param name="Details">Any additional details for the Authenticator</param>
        /// <returns>The registration status</returns>
        public bool IsRegistered(string Name, string Type, object[] Details)
        {
            foreach (IAuthenticator a in this)
            {
                if(a.IsRegistered(Name, Type, Details))
                    return true;
            }

            return false;
        }

        /// <summary>
        /// Attempts to authenticate the named object
        /// </summary>
        /// <param name="Name">The object name</param>
        /// <param name="Details">Any additional details for the Authenticator</param>
        /// <returns>The authentication status</returns>
        public bool Authenticate(string Name, string Type, object[] Details)
        {
            foreach (IAuthenticator a in this)
            {
                if(a.Authenticate(Name, Type, Details))
                    return true;
            }

            return false;
        }

        #endregion
    }
}
