using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Security;

namespace DuffBeer.Security.Key_Stores
{
    public class KeyCache : IKeyManager
    {
        private Dictionary<string, byte[]> cache;

        public KeyCache()
        {
            cache = new Dictionary<string, byte[]>();
        }

        #region IKeyManager Members

        /// <summary>
        /// Retreives a key from the cache
        /// </summary>
        /// <param name="Name">The key to retrieve</param>
        /// <param name="Details">Additional information about the key</param>
        /// <returns>The requested key, or <code>null</code> if not found</returns>
        public byte[] GetKey(string Name, object[] Details)
        {
            if (cache.ContainsKey(Name))
                return cache[Name];
            else
                return null;
        }

        /// <summary>
        /// Sets the key in the cached
        /// </summary>
        /// <param name="Name">The key to change</param>
        /// <param name="Key">The new key</param>
        /// <param name="Details">Additional information about the key</param>
        public void SetKey(string Name, byte[] Key, object[] Details)
        {
            if (cache.ContainsKey(Name))
                cache[Name] = Key;
            else
                cache.Add(Name, Key);
        }

        /// <summary>
        /// Determines if the cache contains the specified key
        /// </summary>
        /// <param name="Name">The key name</param>
        /// <param name="Details">Additional information about the key</param>
        /// <returns>Whether the requested key is cached</returns>
        public bool ContainsKey(string Name, object[] Details)
        {
            return cache.ContainsKey(Name);
        }

        /// <summary>
        /// Removes the key for an object
        /// </summary>
        /// <param name="Name">The key name</param>
        /// <param name="Details">Additional information about the object</param>
        public void RemoveKey(string Name, object[] Details)
        {
            cache.Remove(Name);
        }

        #endregion
    }
}
