using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Security;

namespace DuffBeer.Security.Key_Stores
{
    public sealed class Multi_KeyManager : List<IKeyManager>, IKeyManager
    {
        #region IKeyManager Members

        /// <summary>
        /// Gets the first key for the named object
        /// </summary>
        /// <param name="Name">The object name</param>
        /// <param name="Details">Any additional information for the KeyManager</param>
        /// <returns>The object's key</returns>
        public byte[] GetKey(string Name, object[] Details)
        {
            foreach (IKeyManager k in this)
            {
                byte[] o = k.GetKey(Name, Details);
                if (o != null)
                    return o;
            }

            return null;
        }

        /// <summary>
        /// Sets the key for the named object in the first KeyManager
        /// </summary>
        /// <param name="Name">The object name</param>
        /// <param name="Key">The new key</param>
        /// <param name="Details">Any additional data for the KeyManager</param>
        /// <remarks>Setting the key in the first KeyManager facilitates caching.  It is recommended that the
        /// first KeyManager be a Key Cache.</remarks>
        public void SetKey(string Name, byte[] Key, object[] Details)
        {
            if(this.Count > 0 && this[0] != null)
                this[0].SetKey(Name, Key, Details);
        }

        /// <summary>
        /// Determines if the key manager contains the requested key
        /// </summary>
        /// <param name="Name">The name of the key</param>
        /// <param name="Details">Additional information about the object</param>
        /// <returns>Whether the key is in the key manager</returns>
        public bool ContainsKey(string Name, object[] Details)
        {
            foreach (IKeyManager k in this)
            {
                if(k.ContainsKey(Name, Details))
                    return true;
            }

            return false;
        }

        /// <summary>
        /// Removes the key for an object
        /// </summary>
        /// <param name="Name">The key name</param>
        /// <param name="Details">Additional information about the object</param>
        public void RemoveKey(string Name, object[] Details)
        {
            foreach (IKeyManager k in this)
                k.RemoveKey(Name, Details);
        }

        #endregion
    }
}
