using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Sensors.Interfaces;
using DuffBeer.Sensors.Utility;
using MrHomer.Database;
using MrHomer.Database.Query;
using DuffBeer.Database.StructureTypes;
using DuffBeer.Database;
using MrHomer.Plugins;

namespace DuffBeer.Sensors
{
    [MrHomerPlugin("Name", "Database Cleanup Filter")]
    [MrHomerPlugin("Description", "Cleans up DatabaseFilter's output into nice, simple snapshots.")]
    public class DatabaseCleanupFilter : IDataFilter
    {

        private int dbid;
        public int DBID
        {
            get
            {
                return dbid;
            }
            set
            {
                dbid = value;
            }

        }

        DatabaseManager dm;
        IQueryBuilder qb;
        public DatabaseCleanupFilter()
        {
            dm = DatabaseManager.getInstance();
            qb = dm.GetQueryBuilder(DuffBeerDatabaseConfig.DatabaseName);
        }

        #region IDataFilter Members

        /// <summary>
        /// The counter that determines if the current data should go into the database
        /// </summary>
        private int refreshCount = 0;

        //Delegate and event for notifying the GUI of lcleaned up logged data
        public delegate void CleanDataLoggedNotification(string data);
        public static CleanDataLoggedNotification CleanDataLoggedIntoDBNotification;

        public object Filter(object data)
        {
            DataReading r = data as DataReading;
            
            //check to see if it's been 5 minutes since the last database entry was saved
            refreshCount++;
            if (!(refreshCount * sensor.refresh >= int.Parse(Brewery.getInstance().Config("LogDelay"))))
                return r;
            else
                refreshCount = 0;

            if (CleanDataLoggedIntoDBNotification != null)
                CleanDataLoggedIntoDBNotification("Clean Data logged into DB for sensor: " + r.SensorID);

            SensorsTableEntry lol = SensorsTableEntry.Find(SensorsTableEntry.NameColumn, r.SensorID);
            if (lol != null)
            {
                string cond = qb.And(SnapshotsTableEntry.DurationColumn + "=0", SnapshotsTableEntry.SensorColumn+ "=" + lol.ID);
                string s = qb.SelectWhere(DuffBeerDatabaseConfig.SnapshotsTable, cond, qb.Count("*"));
                DBResults pain = dm.QueryDatabase(DuffBeerDatabaseConfig.DatabaseName, s);
                if (int.Parse(pain[0, 0]) > 100)
                {
                    s = qb.GroupBy(qb.SelectWhere(DuffBeerDatabaseConfig.SnapshotsTable, cond, 
                        qb.Alias(qb.Avg(SnapshotsTableEntry.AverageColumn), "_AVG"),
                        qb.Alias(qb.Min(SnapshotsTableEntry.AverageColumn), "_MIN"),
                        qb.Alias(qb.Max(SnapshotsTableEntry.AverageColumn), "_MAX"),
                        qb.Alias(qb.Sum(SnapshotsTableEntry.AverageColumn), "_SUM"),
                        qb.Alias(qb.Min(SnapshotsTableEntry.TimestampColumn), "_SDATE"),
                        qb.Alias(qb.Max(SnapshotsTableEntry.TimestampColumn), "_EDATE")),
                        SnapshotsTableEntry.SensorColumn);
                    DBResults omg = dm.QueryDatabase(DuffBeerDatabaseConfig.DatabaseName, s);



                    double avg = double.Parse(omg[0, 0]);
                    double min = double.Parse(omg[0, 1]);
                    double max = double.Parse(omg[0, 2]);
                    double tot = double.Parse(omg[0, 3]);
                    DateTime t = DateTime.Parse(omg[0, 4]);
                    DateTime t2 = DateTime.Parse(omg[0, 5]);
                    TimeSpan town = t - t2;
                    town = town.Duration();
                    double len = town.TotalSeconds;

                    s = qb.Delete(DuffBeerDatabaseConfig.SnapshotsTable, cond);
                    dm.QueryDatabase(DuffBeerDatabaseConfig.DatabaseName, s);

                    SnapshotsTableEntry entry = new SnapshotsTableEntry(lol.ID, avg, min, max, tot, t.ToString(), len);
                    entry.Save();
                }
            }
            return r;
        }

        public byte[] SerializeConfiguration()
        {
            return new byte[] { 5,5,5,5 };
        }

        public void DeserializeConfiguration(byte[] data)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public Type InputType
        {
            get { return typeof(DataReading); }
        }

        public Type OutputType
        {
            get { return typeof(DataReading); }
        }

        #endregion

        #region IPlugin Members

        public string Author
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public string Version
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public string Label
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        #endregion

        #region IDataFilter Members

        private Sensor sensor;
        public Sensor associatedSensor
        {
            get
            {
                return sensor;
            }
            set
            {
                sensor = value;
            }
        }

        #endregion
    }
}
