using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Sensors.Interfaces;
using DuffBeer.Sensors.Utility;
using DuffBeer.Database.StructureTypes;
using MrHomer.Plugins;

namespace DuffBeer.Sensors
{
    [MrHomerPlugin("Name", "Database Filter")]
    [MrHomerPlugin("Description", "Puts info in base'o'data.")]
    public class DatabaseFilter : IDataFilter
    {

        private Sensor sensor;
        public Sensor associatedSensor
        {
            get
            {
                return sensor;
            }
            set
            {
                sensor = value;
            }
        }

        private int dbid;
        public int DBID
        {
            get
            {
                return dbid;
            }
            set
            {
                dbid = value;
            }

        }

        /// <summary>
        /// The counter that determines if the current data should go into the database
        /// </summary>
        private int refreshCount = 0;

        //Delegate and event for notifying the GUI of logged data
        public delegate void DataLoggedNotification(string data);
        public static DataLoggedNotification DataLoggedIntoDBNotification;

        public object Filter(object data)
        {
            DataReading r = data as DataReading;

            //check to see if it's been 5 minutes since the last database entry was saved
            refreshCount++;
            if (!(refreshCount * sensor.refresh >= int.Parse(Brewery.getInstance().Config("LogDelay"))))
                return r;
            else
                refreshCount = 0;

            SensorsTableEntry lol = SensorsTableEntry.Find(SensorsTableEntry.NameColumn, r.SensorID);
            if (lol != null)
            {
                //check to see if we already have logged this data before (judging by the timestamp/sensorID)
                SnapshotsTableEntry s = SnapshotsTableEntry.Find(SnapshotsTableEntry.TimestampColumn, r.Time.ToString());
                if (s == null || s.ID == lol.ID)
                {
                    if (DataLoggedIntoDBNotification != null)
                        DataLoggedIntoDBNotification("Data logged into DB for sensor: " + r.SensorID);

                    SnapshotsTableEntry table = new SnapshotsTableEntry(lol.ID, r.Value, r.Value, r.Value, r.Value, r.Time.ToString(), 0);
                    table.Save();
                }
            }
            return r;
            
        }

        public byte[] SerializeConfiguration()
        {
            return new byte[] { 5, 5, 5, 5 };
        }

        public void DeserializeConfiguration(byte[] data)
        { }

        public Type InputType
        {
            get { return typeof(DataReading); }
        }

        public Type OutputType
        {
            get { return typeof(DataReading); }
        }

        #region IPlugin Members

        public string Author
        {
            get { return "Christopher Scalabrini"; }
        }

        public string Version
        {
            get { return "1.0.0.0"; }
        }

        public string Label
        {
            get { return "Throws things into the snapshots database table."; }
        }

        #endregion
    }
}
