using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Plugins;
using DuffBeer.Sensors.Utility;

namespace DuffBeer.Sensors.Interfaces
{
    /// <summary>
    /// This interface is for filtering data.
    /// </summary>
    [MrHomerPlugin("Required", "Name")]
    [MrHomerPlugin("Required", "Description")]
    public interface IDataFilter : IPlugin
    {
        /// <summary>
        /// Sends data to the Filter plugin and then tosses it out.  Do what you will with it.
        /// </summary>
        /// <param name="data">The data</param>
        /// <returns>Nothing, or more data.</returns>
        object Filter(object data);

        /// <summary>
        /// Will serialize the configuration of a sensor so that it may be loaded at a later time.
        /// </summary>
        /// <returns>A byte stream of the serialization information.</returns>
        byte[] SerializeConfiguration();

        /// <summary>
        /// Will reconfigure the ISensor object given a byte stream.
        /// </summary>
        /// <param name="data"></param>
        void DeserializeConfiguration(byte[] data);

        /// <summary>
        /// The type of the input this filter expects.
        /// </summary>
        Type InputType
        { get; }

        /// <summary>
        /// The type of the output this filter produces, if any.
        /// </summary>
        Type OutputType
        { get; }

        /// <summary>
        /// The database ID.
        /// </summary>
        int DBID
        {
            get;
            set;
        }

        /// <summary>
        /// The sensor this filter is attached to
        /// </summary>
        Sensor associatedSensor
        {
            get;
            set;
        }
    }
}
