using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Plugins;
using DuffBeer.Sensors.Utility;

namespace DuffBeer.Sensors.Interfaces
{
    /// <summary>
    /// Provides an interface to a reporting device
    /// </summary>
    [MrHomerPlugin("Required", "Name")]
    [MrHomerPlugin("Required", "Make")]
    [MrHomerPlugin("Required", "Model")]
    [MrHomerPlugin("Required", "Description")]
    public interface ISensor : IPlugin
    {
        /// <summary>
        /// Requests data from the reporting device
        /// </summary>
        /// <returns>An instance an object representing the data reading.</returns>
        object RequestData();

        /// <summary>
        /// Gets the unit of measurement the reporting device is using
        /// </summary>
        /// <returns>The units the sensor is reporting (kWh, BTU, etc).</returns>
        string GetUnits();

        /// <summary>
        /// Sets the name of the sensor.
        /// </summary>
        /// <param name="name">The name of the sensor.</param>
        void SetName(string name);

        /// <summary>
        /// Will serialize the configuration of a sensor so that it may be loaded at a later time.
        /// </summary>
        /// <returns>A byte stream of the serialization information.</returns>
        byte[] SerializeConfiguration();

        /// <summary>
        /// Will reconfigure the ISensor object given a byte stream.
        /// </summary>
        /// <param name="data"></param>
        void DeserializeConfiguration(byte[] data);

        Type ReturnType
        { get; }

    }
}
