using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Plugins;

namespace DuffBeer.Sensors
{
    /// <summary>
    /// Sensor information class.
    /// </summary>
    public class SensorInformation
    {
        private string name;
        private string make;
        private string model;
        private string description;
        private int refresh;
        private string localName;

        /// <summary>
        /// The name of the sensor/plugin.
        /// </summary>
        public string Name
        { get { return name; } }
        /// <summary>
        /// The make of the sensor.
        /// </summary>
        public string Make
        { get { return make; } }
        /// <summary>
        /// The model of the sensor.
        /// </summary>
        public string Model
        { get { return model; } }
        /// <summary>
        /// The plugin's description.
        /// </summary>
        public string Description
        { get { return description; } }
        /// <summary>
        /// The plugin's refresh rate
        /// </summary>
        public int Refresh
        { get { return refresh; } }
        /// <summary>
        /// The local name of the sensor/plugin.
        /// </summary>
        public string LocalName
        { get { return localName; } }

        /// <summary>
        /// The main constructor.
        /// </summary>
        /// <param name="myName">The name.</param>
        /// <param name="myMake">The make.</param>
        /// <param name="myModel">The model.</param>
        /// <param name="myDesc">The description.</param>
        public SensorInformation(string myName, string myMake, string myModel, string myDesc, int myRefresh, string myLocalName)
        {
            name = myName;
            make = myMake;
            model = myModel;
            description = myDesc;
            refresh = myRefresh;
            localName = myLocalName;
        }

        /// <summary>
        /// A cooler constructor.  Autopopulates based on a plugin.
        /// </summary>
        /// <param name="t">The type.</param>
        public SensorInformation(Type t)
        {
            PluginManager pm = PluginManager.getInstance();
            if(!pm.EnforceAttributes("Required", t))
            {
                throw new SensorException("Did not include all attributes.");
            }
            List<MrHomerPluginAttribute> list = pm.getAttributes(t);
            foreach (MrHomerPluginAttribute att in list)
            {
                if (att.Name.ToLower() == "name")
                    name = att.Value;
                if (att.Name.ToLower() == "make")
                    make = att.Value;
                if (att.Name.ToLower() == "model")
                    model = att.Value;
                if (att.Name.ToLower() == "description")
                    description = att.Value;
            }

            localName = "";
            refresh = 0;
        }
    }
}
