using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MrHomer.Security;
using DuffBeer.Security;
using DuffBeer.Security.Cryptography;
using MrHomer.Database.Query;
using DuffBeer.Security.Authentication;
using MrHomer.Database;
using DuffBeer.Security.Key_Stores;
using DuffBeer;
using DuffBeer.Sensors;
using AsyncSockets;
using System.Net;
using MrHomer.Networking;
using System.Threading;

namespace DuffBeerDriver
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            Thread.CurrentThread.Name = "Main Form";
        }

        internal delegate void DelayedMessage(string message);

        private void LogMessage(string message)
        {
            if (txtLog.IsDisposed) return;
            if (txtLog.InvokeRequired)
            {
                txtLog.BeginInvoke(new DelayedMessage(LogMessage), message);
            }
            else
            {
                txtLog.Text += message;
                txtLog.SelectionStart = txtLog.Text.Length;
                txtLog.SelectionLength = 0;
                txtLog.ScrollToCaret();
            }
        }

        private string ArrayToString(string name, Array a)
        {
            string result = name + " { ";
            for (int i = 0; i < a.Length - 1; i++)
                result += a.GetValue(i).ToString() + ", ";

            result += a.GetValue(a.Length - 1) + " }\n";

            return result;
        }

        private bool ArrayEquals(Array a, Array b)
        {
            if (a == null || b == null) return false;
            if (a.Length != b.Length) return false;

            try
            {
                for (int i = 0; i < a.Length; i++)
                    if (!a.GetValue(i).Equals(b.GetValue(i)))
                        return false;
            }
            catch
            {
                return false;
            }

            return true;
        }

        private bool ConnectionTest()
        {
            try
            {
                LogMessage("Testing ConnectionManager\n");
                ConnectionManager me = ConnectionManager.getInstance();
                me.SetRequestPacket(new byte[] { 1, 3, 3, 7 });
                me.SetCredentials(new byte[] { 1, 3, 3, 7 });
                me.SetDropout(1000, true);
                LogMessage("Listening for moar connekshuns...\n");
                me.ListenForConnection(1337, true);
                LogMessage("maeking wun moar connekshun...\n");
                me.MakeConnection("Me! Lol.", "127.0.0.1", 1337, true);
                LogMessage("Nighty night...\n");
                LogMessage("O hai I is awake...\n");
                while (me.GetAllInactiveConnections().Count == 0) ;
                foreach (KeyValuePair<string, byte[]> lol in me.GetAllInactiveConnections())
                {
                    LogMessage("I has a " + BitConverter.ToInt32(lol.Value,0).ToString() + ". Let me show you it.  It's from " + lol.Key + "!" + "\n");
                    me.ActivateConnection(lol.Key, lol.Key + "PENIS! Lol.");
                }

                me.SendData("Me! Lol.", new byte[] { 6, 6, 6 });
                LogMessage("Sending stuff!!!\n");
                while (me.GetData(false).Count == 0) ;
                foreach (byte[] poon in me.GetData(true))
                {
                    LogMessage("I got a " + ArrayToString("", poon) + "\n");
                    return true;
                }
                return false;
            }
            catch
            {
                return false;
            }
        }

        private bool SensorTest()
        {
            LogMessage("Testing Sensors\n");

            LogMessage("Checking if sensors loaded ... ");
            List<SensorInformation> sensors = SensorManager.getInstance().GetSensorTypeInformation();

            if (sensors.Count > 0)
            {
                LogMessage(" OK!\n");
                foreach (SensorInformation i in sensors)
                    LogMessage("Sensor: " + i.Name + "\n");
            }
            else
            {
                LogMessage("Failed!\n");
                return false;
            }

            return true;
        }

        private bool CryptoTest()
        {
            LogMessage("Testing Encryption\n");
            SecurityManager sm = SecurityManager.getInstance();
            byte[] cypherKey = new byte[] { 15, 7, 9, 18, 6 };

            LogMessage(ArrayToString("Key", cypherKey));

            string m = "The quick brown fox jumped over the lazy dog.";
            byte[] plain = Encoding.UTF8.GetBytes(m);

            //LogMessage(ArrayToString("Plaintext", plain));
            LogMessage("  Plain Text is: " + m + "\n");


            byte[] enc = sm.Encrypt(plain, cypherKey, null);
            string m_enc = Encoding.UTF8.GetString(enc);

            //LogMessage(ArrayToString("Cyphertext", enc));
            LogMessage("Encoded Text is: " + m_enc + "\n");

            byte[] dec = sm.Decrypt(enc, cypherKey, null);
            string m_dec = Encoding.UTF8.GetString(dec);

            ///LogMessage(ArrayToString("Plaintext", dec));
            LogMessage("Decoded Text is: " + m_dec + "\n");

            if (ArrayEquals(plain, dec))
            {
                LogMessage("Texts match!\n");
                return true;
            }
            else
            {
                LogMessage("Texts don't match!\n");
                return false;
            }
        }

        private bool AuthTest()
        {
            bool Win = true;
            SecurityManager sm = SecurityManager.getInstance();
            LogMessage("Testing Authentication\n");

            LogMessage("Testing Unauthorized User ... ");
            if (!sm.Authenticate("FakeUser", "User", new object[] { new byte[] { } }))
            {
                LogMessage("Rejected\n");
            }
            else
            {
                LogMessage("Authenticated\n");
                Win = false;
            }

            LogMessage("Testing Authorized User ... ");
            if (sm.Authenticate("TestUser", "User", new object[] { Encoding.UTF8.GetBytes("AAAA") }))
            {
                LogMessage("Authenticated\n");
            }
            else
            {
                LogMessage("Rejected\n");
                Win = false;
            }

            return Win;
        }

        private bool KeyTest()
        {
            bool Win = true;
            SecurityManager sm = SecurityManager.getInstance();
            LogMessage("Testing Key Management\n");

            LogMessage("Checking for existing key for TestKey ... ");

            if (sm.ContainsKey("TestKey", null))
            {
                LogMessage("Found\n");
            }
            else
            {
                LogMessage("Not Found\n");
                LogMessage("Setting key for TestKey ... ");
                sm.SetKey("TestKey", Encoding.UTF8.GetBytes("ABCD"), null);
                LogMessage("Done.\n");
            }

            LogMessage("Checking key for TestKey ... ");
            byte[] key = sm.GetKey("TestKey", null);
            if (ArrayEquals(key, Encoding.UTF8.GetBytes("ABCD")))
            {
                LogMessage("Match!\n");
            }
            else
            {
                LogMessage("Incorrect Key\n");
                Win = false;
            }

            LogMessage("Removing Key ... ");
            sm.RemoveKey("TestKey", null);
            LogMessage("Done\n");

            return Win;
        }

        private bool QueryBuilderTest()
        {
            LogMessage("Testing Query Builder\n");
            string Expected, Actual;
            bool Win = true;
            QueryBuilder q = new QueryBuilder();

            LogMessage("Testing SELECT * WHERE ... ");
            Expected = "SELECT * FROM SomeTable WHERE Condition=True";
            Actual = q.SelectWhere("SomeTable", "Condition=True");
            LogMessage((Expected == Actual) ? "PASS!\n" : "FAIL!\n");
            Win = Win && (Expected == Actual);

            LogMessage("Testing SELECT [columns] WHERE ... ");
            Expected = "SELECT Col1, Col2 FROM SomeTable WHERE Condition=True";
            Actual = q.SelectWhere("SomeTable", "Condition=True", "Col1", "Col2");
            LogMessage((Expected == Actual) ? "PASS!\n" : "FAIL!\n");
            Win = Win && (Expected == Actual);

            LogMessage("Testing INSERT ... ");
            Expected = "INSERT INTO SomeTable (Key) VALUES ('Value');";
            Actual = q.Insert("SomeTable", new KeyValuePair<string, string>("Key", "Value"));
            LogMessage((Expected == Actual) ? "PASS!\n" : "FAIL!\n");
            Win = Win && (Expected == Actual);

            LogMessage("Testing UPDATE ... ");
            Expected = "UPDATE SomeTable SET Key='Value' WHERE Condition=True;";
            Actual = q.Update("SomeTable", "Condition=True", new KeyValuePair<string, string>("Key", "Value"));
            LogMessage((Expected == Actual) ? "PASS!\n" : "FAIL!\n");
            Win = Win && (Expected == Actual);

            return Win;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            /*
            LogMessage("Doing Setup\n");
            Brewery.getInstance();

            LogMessage("** Starting Sensor Manager Tests **\n");

            if (SensorTest())
                LogMessage("[PASSED] Sensor Manager\n");
            else
                LogMessage("[FAILED] Sensor Manager\n");

            LogMessage("** Starting Communication Tests **\n");

            if (ConnectionTest())
                LogMessage("[PASSED] Connection Manager\n");
            else
                LogMessage("[FAILED] Connection Manager\n");

            LogMessage("** Starting Database Manager Tests **\n");

            if (QueryBuilderTest())
                LogMessage("[PASSED] Query Builder\n");
            else
                LogMessage("[FAILED] Query Builder\n");

            LogMessage("** Starting Security Manager Tests **\n");

            //SecurityManager.getInstance(new DatabaseAuth(), new NRotN_Crypto(), new DBKeyStore());
            //LogMessage("Got a security manager\n");

            if (CryptoTest())
                LogMessage("[PASSED] Encryption\n");
            else
                LogMessage("[FAILED] Encryption\n");

            if (AuthTest())
                LogMessage("[PASSED] Authentication\n");
            else
                LogMessage("[FAILED] Authentication\n");

            if (KeyTest())
                LogMessage("[PASSED] Key Management\n");
            else
                LogMessage("[FAILED] Key Management\n");
            */

            SystemTest.Logger = LogMessage;

            Thread t = new Thread(new ThreadStart(SystemTest.Run));
            t.Name = "Main";
            t.Start();
        }
    }
}