using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Networking;
using MrHomer.Database;
using MrHomer.Plugins;
using MrHomer.Security;
//using MrHomerPluginLibrary.PluginInterfaces;
using System.Reflection;
using DuffBeerPluginLibrary.Interfaces;

namespace MrHomerDriver
{
    /// <summary>
    /// Note, not a real class!  This is just a pile of examples from the SAD.
    /// </summary>
    class Samples
    {
        public void ConnectionManagerExamples()
        {
//*********************************************************

//Getting a connection manager
ConnectionManager cm = ConnectionManager.getInstance();

//*********************************************************

//Required initialization.  3 method calls.
//Set the credentials first.  It's like your system's
//identifier, or signature.

//Set the request packet second.  It must be equal on
//any two systems communicating.  It ensures valid
//programs are communicating.

//Set the dropout.  Connections that don't receive
//credentials will automatically drop after the
//timeout if it's true.  Otherwise, they will attempt
//to get credentials forever.  It is recommended to
//set it to true.

//Also, attempted connections to unresolved host
//names will drop after the timeout, no matter what.
byte[] credentials = new byte[0];
    //Set to something meaningful
byte[] request = new byte[0];
    //Set to something meaningful
uint timeout = 10000;
bool drop = true;
cm.SetCredentials(credentials);
cm.SetRequestPacket(request);
cm.SetDropout(timeout, drop);

//*********************************************************

//Listening on a port
//Note, the second parameter must be "true" for now.
//cm.ListenForConnection(port number, using IPV4);

//To stop listening, call StopListening.
cm.ListenForConnection(1337, true);
cm.StopListening(1337);

//*********************************************************

//Connecting to a remote lost
//Note, last parameter must be "true" for now.
//cm.MakeConnection(Connection Label, Host, Port, using IPV4);
cm.MakeConnection("MyConnection", "192.168.200.35", 65535, true);

//*********************************************************

//Getting information about the current state.

//Active connections are successful connections you initiate.

//Inactive connections are successful connections made on listening ports.
//Note that inactive connections also return a the connecting computer's
//credential packet that was set earlier.

//Failed connections are connections you initiate that failed to connect.
//Dropped connections are formerly active or inactive that dropped without
//user action.  They both require a parameter.  True to delete the data
//after its returned, false to retain it for future reference.
cm.GetAllActiveConnections();
cm.GetAllInactiveConnections();
cm.GetDroppedConnections(false);
cm.GetFailedConnections(false);
cm.GetListeningPorts();

//*********************************************************

//Activating an inactive connection can be done like so.
string oldname = "unknown0";
string newname = "MyOtherConnection";
cm.ActivateConnection(oldname, newname);

//*********************************************************

//To send data, call SendData.
string connection = "MyConnection";
byte[] data = new byte[0];
    //Set to something meaningful.
cm.SendData(connection, data);

//*********************************************************

//To receive data, call GetData.  It has two overloads.
//One accepts a connection name and a clear flag.  The
//other accepts only a clear flag.  Set to true, the clear
//flag will clear all data that was returned from the cache.
//Without a name, it returns all data on all connections that
//has been received.  With a name, it returns only data that
//connection has sent.

string rConnection = "MyOtherConnecton";
bool ClearFlag = false;
cm.GetData(rConnection, ClearFlag);
cm.GetData(ClearFlag);

//*********************************************************

//To terminate an active connection, call KillActiveConnection.
//To terminate an inactive connection, call KillInactiveConnection.
string active = "MyConnection";
string inactive = "unknown0";
cm.KillActiveConnection(active);
cm.KillInactiveConnection(inactive);


        }


        public void PacketSamples()
        {

//*********************************************************

//Creating a new packet from your own data is as simple as
//calling the constructor with information.
string sender = "Me!";
string receiver = "You!";
string subject = "Hi!";
IHSerializable body = new StringData("How are you?");
Packet p = new Packet(sender, receiver, subject, body);

//*********************************************************

//Turning a packet into a byte stream is as easy as calling
//Serialize.

byte[] bytes = p.Serialize();

//*********************************************************

//Turning a byte stream into a  packet can be done by
//creating a new packet object.

Packet newPacket = new Packet(bytes);




        }

        public void NetworkCrossClassSamples()
        {
            //*********************************************************

//The Packet class can be used to create the byte streams
//needed by ConnectionManager.

ConnectionManager cm = ConnectionManager.getInstance();
Packet credentials = new Packet("Chris", 
                            "You",
                            "My Credentials", 
                            new StringData("Let me show you them"));
Packet request = new Packet("Me",
                            "You",
                            "Request",
                            new StringData("Plz?"));
cm.SetCredentials(credentials.Serialize());
cm.SetRequestPacket(request.Serialize());

//*********************************************************

//It can also be used to send information, and to debug it
//when it is received.

Packet message = new Packet("Brian",
                        "Bob",
                        "Text",
                        new StringData("S'up."));
cm.SendData("BobConnection", message.Serialize());

List<byte[]> data = cm.GetData("BobConnection", false);
Packet incoming = new Packet(data[0]);

        }

        public void PluginManagerSamples()
        {
//*********************************************************

//Getting the PluginManager is as easy as calling its
//Singleton method.
PluginManager pm = PluginManager.getInstance();

//*********************************************************

//To load plugin types, call this.
pm.LoadInterfaceTypes("myDLL.dll");

//*********************************************************

//To load actual plugins once types are loaded, call this.
pm.LoadPluginTypes("myOtherDLL.dll");

//*********************************************************

//To get a list of all plugin types loaded, call this.
pm.getInterfaces();

//*********************************************************

//To get a set of plugins given a type, call this.
Type IPluginType = typeof(ISensor);
List<Type> plugins = pm.getPlugins(IPluginType);

//*********************************************************

//To create an instance of a plugin once you have its type,
//call this.
Type plugintype = plugins[0];
Assembly asm = Assembly.GetExecutingAssembly();
ISensor myPlugin;
myPlugin = asm.CreateInstance(plugintype.FullName)as ISensor;

        }

        public void DatabaseSamples()
        {
// Getting a database manager
DatabaseManager dm = DatabaseManager.getInstance();

//Connecting to a database
dm.ConnectDatabase("Name", "DB_Type", "dbserver:1234", "User", "Password", "MyDB", null);

// Running a query
DBResults r = dm.QueryDatabase("Name", "SELECT * FROM SampleTable WHERE Condition = True;");

// Using the results
foreach(Dictionary<string, object> row in r)
    Console.WriteLine("Row: " + r.ToString());

// Closing a connection
dm.DisconnectDatabase("Name");
        }

        public void SecuritySamples()
        {
            IAuthenticator Auth;
            ICryptographer Crypto;
            IKeyManager KeyMan;
            byte[] message;

// Getting a Security Manager
SecurityManager sm = SecurityManager.getInstance(Auth, Crypto, KeyMan);

// Authenticating a User
sm.Authenticate("User", null);

// Finding a Host Key
object[] key = sm.GetKey("Host", null);

// Encrypting a Packet
byte[] packet = sm.Encrypt(message, key);

// Decrypting a Packet
byte[] data = sm.Decrypt(message, key);
        }
    }
}
