using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MrHomer.Plugins;
using MrHomer.Database;
using DuffBeer.Sensors;
using DuffBeerDriver.FakePlugins;
using DuffBeerDriver.Database;

namespace DuffBeerDriver
{
    public partial class SensorManagerTestPanel : Form
    {
        SensorManager sm;
        PluginManager pm;
        DatabaseManager dm;

        List<string> lines;
        public SensorManagerTestPanel()
        {
            InitializeComponent();
        }

        private void SensorManagerTestPanel_Load(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
        }

        public void update()
        {
            console.Lines = lines.ToArray();
            console.SelectionStart = console.Text.Length;
            console.SelectionLength = 0;
            console.ScrollToCaret();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            this.Focus();
            MyLoad(openFileDialog1.FileName);
        }

        public void MyLoad(string s)
        {
            lines = new List<string>();
            
            sm = SensorManager.getInstance();
            lines.Add("Obtained Sensor Manager.");
            update();

            pm = PluginManager.getInstance();
            lines.Add("Obtained Plugin Manager.");
            update();

            dm = DatabaseManager.getInstance();
            lines.Add("Obtained Database Manager.");
            update();

            dm.RegisterType("Fake", new FakeDatabaseFactory(), null);
            lines.Add("Registered 'Fake' Database Type.");
            update();

            dm.ConnectDatabase("lol", "Fake", "", "", "", "", null);
            lines.Add("Connected to Fake database 'lol'.");
            update();

            try
            {
                pm.LoadInterfaceTypes(s);
                lines.Add("Interfaces loaded from DLL.");
                update();
            }
            catch (Exception e)
            {
                lines.Add(e.Message);
                update();
                return;
            }

            pm.LoadInterfaceTypes(System.Reflection.Assembly.GetExecutingAssembly());
            lines.Add("Interfaces loaded from current assembly.");
            update();

            try
            {
                pm.LoadPluginTypes(s);
                lines.Add("Plugins loaded from DLL.");
                update();
            }
            catch (Exception e)
            {
                lines.Add(e.Message);
                update();
                return;
            }

            pm.LoadPluginTypes(System.Reflection.Assembly.GetExecutingAssembly());
            lines.Add("Plugins loaded from current assembly.");
            update();

            try
            {
                sm.LoadSensorsAndFilters();
                lines.Add("Sensor and Filter Plug-ins loaded.");
            }
            catch (SensorException ex)
            {
                lines.Add(ex.Message);
                update();
                return;
            }

            btnGetSensorTypes_Click(null, null);
            btnGetFilterTypes_Click(null, null);


        }

        private void btnMake_Click(object sender, EventArgs e)
        {
            lines.Add("Attempting to create a sensor...");
            update();

            try
            {
                if (sm.InstantiateSensor(txtMake1.Text, txtMake2.Text, int.Parse(txtMake3.Text)))
                    lines.Add("Successfully added sensor!");
                else
                    lines.Add("Failure.");
            }
            catch (Exception ex)
            {
                lines.Add(ex.Message);
            }
            update();
        }

        private void btnAddFilter_Click(object sender, EventArgs e)
        {
            lines.Add("Attempting to add filter...");
            update();

            try
            {
                if (sm.AddFilter(txtFilter1.Text, txtFilter2.Text))
                    lines.Add("Successfully added filter!");
                else
                    lines.Add("Failure.");
            }
            catch (Exception ex)
            {
                lines.Add(ex.Message);
            }
            update();
        }

        private void btnGetFilter_Click(object sender, EventArgs e)
        {
            lines.Add("Attempting to get filters...");
            update();

            try
            {
                List<FilterInformation> list = sm.GetFiltersOnSensor(txtFilter1.Text);
                lines.Add("Sensor " + txtFilter1.Text + " contains filters:");
                foreach (FilterInformation fi in list)
                {
                    lines.Add("--" + fi.Name);
                }
            }
            catch (Exception ex)
            {
                lines.Add(ex.Message);
            }
            update();
        }

        private void btnClearFilter_Click(object sender, EventArgs e)
        {
            lines.Add("Attempting to clear filters...");
            update();

            try
            {
                if (sm.ClearFilters(txtFilter1.Text))
                    lines.Add("Successfully cleared filters!");
                else
                    lines.Add("Failure.");
            }
            catch (Exception ex)
            {
                lines.Add(ex.Message);
            }
            update();
        }

        private void btnStartSensor_Click(object sender, EventArgs e)
        {
            lines.Add("Attempting to start sensor...");
            update();

            try
            {
                if (sm.StartSensor(txtSensor1.Text))
                    lines.Add("Successfully started sensor!");
                else
                    lines.Add("Failure.");
            }
            catch (Exception ex)
            {
                lines.Add(ex.Message);
            }
            update();
        }

        private void btnStopSensor_Click(object sender, EventArgs e)
        {
            lines.Add("Attempting to stop sensor...");
            update();

            try
            {
                if (sm.StopSensor(txtSensor1.Text))
                    lines.Add("Successfully stopped sensor!");
                else
                    lines.Add("Failure.");
            }
            catch (Exception ex)
            {
                lines.Add(ex.Message);
            }
            update();
        }

        private void btnSetRefresh_Click(object sender, EventArgs e)
        {
            lines.Add("Attempting to set sensor refresh rate...");
            update();

            try
            {
                if (sm.SetRefresh(txtSensor1.Text, int.Parse(txtSensor2.Text)))
                    lines.Add("Successfully set sensor refresh rate!");
                else
                    lines.Add("Failure.");
            }
            catch (Exception ex)
            {
                lines.Add(ex.Message);
            }
            update();
        }

        private void btnGetSensorTypes_Click(object sender, EventArgs e)
        {
            lines.Add("Available Sensor Types:");
            foreach (SensorInformation si in sm.GetSensorTypeInformation())
            {
                lines.Add("--" + si.Name);
                lines.Add("----" + si.Make);
                lines.Add("----" + si.Model);
                lines.Add("----" + si.Description);
            }
            update();
        }

        private void btnGetFilterTypes_Click(object sender, EventArgs e)
        {
            lines.Add("Available Filter Types:");
            foreach (FilterInformation fi in sm.GetFilterTypeInformation())
            {
                lines.Add("--" + fi.Name);
                lines.Add("----" + fi.Description);
            }
            update();
        }

        private void btnGetCreated_Click(object sender, EventArgs e)
        {
            lines.Add("Created Sensors:");
            foreach (string fi in sm.GetCreatedSensors())
            {
                lines.Add("--" + fi);
            }
            update();
        }

        private void btnStartAll_Click(object sender, EventArgs e)
        {
            lines.Add("Starting all sensors.");
            sm.StartAllSensors();
            update();
        }

        private void btnStopAll_Click(object sender, EventArgs e)
        {
            lines.Add("Stopping all sensors.");
            sm.StopAllSensors();
            update();
        }

        private void btnDatabse_Click(object sender, EventArgs e)
        {
            IDatabase data = dm.FindDatabase("lol");
            DBResults r = data.Query("get");
            for (int i = 0; i < r.Length(); i++)
            {
                foreach (object o in r[i].Values)
                {
                    double d = (double)o;
                    lines.Add(d.ToString());
                }
            }
            update();
        }
    }
}