using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer;
using DuffBeer.Communication;
using MrHomer.Networking;
using MrHomer.Security;
using MrHomer.Plugins;
using DuffBeer.Messages.MessageTypes;
using System.Threading;
using DuffBeerDriver.FakePlugins;
using DuffBeer.Security.Cryptography;
using DuffBeer.Sensors;
using DuffBeer.Messages.Serialization;
using MrHomer;

namespace DuffBeerDriver
{
    static class SystemTest
    {
        private static string Connection = "Test Connection";
        private static volatile CommunicationManager cm;
        private static ErrorManager em;
        private static ConnectionManager mm;
        private static SensorManager sm;
        public static MainForm.DelayedMessage Logger;
        private static Thread t;

        private static void Log(string message) {
            Logger(message);
        }

        private static void ErrorLog(Exception e)
        {
            Log(e.ToString() + "\n");
        }

        private static void MsgThreader()
        {
            while(cm.HasEvents)
            {
                try
                {
                    cm.DoEvents();
                    em.DumpErrors(new ErrorManager.ErrorPrinter(SystemTest.ErrorLog));
                    Thread.Sleep(100);
                }
                catch (ThreadAbortException)
                {
                    return;
                }
            }
            Log(">> Event Loop Finished\n");
        }

        /// <summary>
        /// Runs the tests
        /// </summary>
        public static void Run()
        {

            Log("Ready to run communication tests\n");
            Log("Set Up\n");
            SetUp();
            Log("Ready\n");
            
            Connect();
            ConfigGetter();
            SensorList();
            //SensorData();
            SystemList();
            UserTests();

            Log("Complete\n");
            Finish();
            Logger("Done now.\n");
        }

        /// <summary>
        /// Sets up the connection manager and whatnot
        /// </summary>
        private static void SetUp()
        {
           //Thread.CurrentThread.Name = "System Tester";

            SecurityManager.getInstance(new SuperAuth(), new NRotN_Crypto(), new SuperKey());
            Brewery.getInstance();
            cm = CommunicationManager.getInstance();
            mm = ConnectionManager.getInstance();
            em = ErrorManager.getInstance();
            sm = SensorManager.getInstance();

            SensorsStuff();
            
            t = new Thread(new ThreadStart(SystemTest.MsgThreader));
            t.Name = "Events Loop";
            t.Start();
        }

        /// <summary>
        /// Completes the whatever
        /// </summary>
        private static void Finish()
        {
            //Brewery.getInstance().Dispose();
        }

        private static void Connect()
        {
            Log("Trying to connect...\n");
            ITask conn = new ConnectionTask();
            conn.InitializeTask(Connection, new object[] { "127.0.0.1", int.Parse(Brewery.getInstance().Config("Port")), true });
            cm.AddTask(conn);

            while (mm.GetAllActiveConnections().Count == 0) ;
            Log("Connected!\n");

            Thread.Sleep(0);
        }

        private static void ConfigGetter()
        {
            Log("Attempting configuration retrieval...\n");
            string varName = "TestB";
            
            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(ConfigurationValueRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Brewery.getInstance().AddVariable(varName);

            Log("Sending out the request...\n");
            Packet p = new Packet("TestA", "TestB", subject, new StringData("BreweryName"));
            ITask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            Log("Awaiting Response...\n");
            while (!Brewery.getInstance().IsVariableSet(varName))
                Thread.Sleep(500);

            object o = Brewery.getInstance().PopVariable(varName);
            string s = (string)o;

            Log("Got it! It was " + s + "\n");
        }

        private static void SensorList()
        {
            Log("Trying to find available sensors...\n");

            string varName = "Test Connection_SensorList";

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(SensorListNumberRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Brewery.getInstance().AddVariable(varName);

            Log("Sending out the request...\n");
            Packet p = new Packet("TestA", "TestB", subject, new StringData(""));
            ITask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            Log("Awaiting Response...\n");
            while (!Brewery.getInstance().IsVariableSet(varName))
                Thread.Sleep(500);

            object o = Brewery.getInstance().PeekVariable(varName);
            List<String> sensors = (List<String>)o;

            foreach (String s in sensors)
                Log("Sensor That Brewery Returned: " + s + "\n");

            Log("Got it! There are " + sensors.Count + " sensors.\n");
        }

        private static void SensorsStuff()
        {
            Log("Setting up local sensors...");

            string sName = sm.GetSensorTypeInformation()[0].Name;
            List<FilterInformation> f = sm.GetFilterTypeInformation().FindAll(new Predicate<FilterInformation>(FilterFinder));
            string fName = f[0].Name;
            string fName2 = f[1].Name;
            sm.InstantiateSensor("Whatever", "Captain Planet", sName, 30);
            sm.AddFilter("Whatever", "Captain Rock Mass", fName);
            sm.AddFilter("Whatever", "Captain Stone Mass", fName);
            sm.AddFilter("Whatever", "Captain Rock Ball", fName);
            sm.AddFilter("Whatever", "Captain Stone Ball", fName);
            sm.AddFilter("Whatever", "...COCK!", fName2);
            sm.StartAllSensors();
            Thread.Sleep(2500);
            sm.StopAllSensors();

            Log("OK!\n");
        }

        private static bool FilterFinder(FilterInformation f)
        {
            return f.Name != "Composite Data Filter";
        }

        private static void SensorData()
        {
            Log("Looking for the latest sensor data...\n");

            object o = Brewery.getInstance().PopVariable("Test Connection_SensorList");
            List<SensorInformation> sensors = (List<SensorInformation>)o;

            string varName = "Test Connection_" + sensors[0].LocalName + "_Data";

            if (sensors.Count == 0)
            {
                Log("No Sensors =(\n");
                return;
            }

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(SensorDataRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Brewery.getInstance().AddVariable(varName);

            Log("Sending out the request...\n");
            Packet p = new Packet("TestA", "TestB", subject, new StringData(sensors[0].LocalName));
            ITask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            Log("Awaiting Response...\n");
            while (!Brewery.getInstance().IsVariableSet(varName))
                Thread.Sleep(500);

            object x = Brewery.getInstance().PopVariable(varName);
            Dictionary<string, string> sd = (Dictionary<string, string>)x;

            Log("Got it! The Average is " + sd["Avg"] + "\n");
        }

        private static void SystemList()
        {
            Log("Trying to find available systems...\n");

            string varName = "Test Connection_SystemList";

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(RegisteredSystemNumberRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Brewery.getInstance().AddVariable(varName);

            Log("Sending out the request...\n");
            Packet p = new Packet("TestA", "TestB", subject, new StringData(""));
            ITask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            Log("Awaiting Response...\n");
            while (!Brewery.getInstance().IsVariableSet(varName))
                Thread.Sleep(500);

            object o = Brewery.getInstance().PeekVariable(varName);
            StringListData systems = (StringListData)o;

            Log("Got it! There are " + systems.Count + " systems below " + "Test Connection" + ".\n");
        }

        public static void UserTests()
        {
            Log("Checking user recognition...\n");
            string varName = "TestConnection_UserReq";

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(UserRecognitionRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Brewery.getInstance().AddVariable(varName);

            UserInformation i = new UserInformation("UltraUser!", "", false);

            Log("Sending out the request...\n");
            Packet p = new Packet(Brewery.getInstance().UPRN, varName, subject, i);
            ITask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            Log("Awaiting Response...\n");
            while (!Brewery.getInstance().IsVariableSet(varName))
                Thread.Sleep(500);

            object o1 = Brewery.getInstance().PopVariable(varName);
            bool win1 = (bool)o1;

            if (win1)
            {
                Log("User Recognized\n");
            }
            else
                Log("Unknown User\n");

            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(UserRegistrationRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Brewery.getInstance().AddVariable(varName);

            Log("Trying to register...\n");
            p = new Packet(Brewery.getInstance().UPRN, varName, subject, i);
            task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            Log("Awaiting Response...\n");
            while (!Brewery.getInstance().IsVariableSet(varName))
                Thread.Sleep(500);

            object o2 = Brewery.getInstance().PopVariable(varName);
            bool win2 = (bool)o2;

            if (win2)
            {
                Log("User Regetistered\n");
            }
            else
            {
                Log("Failed to register\n");
            }


            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(UserDeregistrationMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Brewery.getInstance().AddVariable(varName);

            Log("And, removing again...\n");
            p = new Packet(Brewery.getInstance().UPRN, varName, subject, i);
            task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);

            Thread.Sleep(0);
            Log("User Deregetistered\n");
        }
    }
}
