using System;
using System.Collections.Generic;
using System.Text;

namespace MrHomer.Database
{
    internal class DatabaseAFactory
    {

        private Dictionary<string, IDatabaseFactory> reference;

        /// <summary>
        /// Constructor.  Will never be seen outside of MrHomer.
        /// </summary>
        internal DatabaseAFactory()
        {
            reference = new Dictionary<string, IDatabaseFactory>();
        }


        /// <summary>
        /// Registers a type of factory with the Abstract Factory.
        /// </summary>
        /// <param name="name">The new type's name.</param>
        /// <param name="factory">The factory object.</param>
        internal void RegisterType(string name, IDatabaseFactory factory)
        {
            if (reference.ContainsKey(name))
            {
                return;
            }
            reference.Add(name, factory);
        }

        /// <summary>
        /// Makes a database.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <param name="arguments">Args</param>
        /// <returns>The IDatabase object.</returns>
        internal IDatabase GetDatabase(string type, object[] arguments)
        {
            if (!reference.ContainsKey(type))
            {
                return null;
            }
            return reference[type].GetDatabase(arguments);
        }
    }
}
