using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Plugins;

namespace MrHomer.Database
{
    [MrHomerPlugin("Required", "Type")]
    public interface IDatabase : IPlugin
    {
        /// <summary>
        /// Initiates a connection to the database
        /// </summary>
        /// <param name="dataSource">A string identifying the server</param>
        /// <param name="username">The user name for the connection</param>
        /// <param name="password">The password for the connection</param>
        /// <param name="database">The database to use</param>
        /// <returns>True if the connection was established; False if an error occured</returns>
        /// <example>Connect("localhost:5555", "JoeUser", "MyPass", "JoesData")</example>
        bool Connect(string dataSource, string username, string password, string database);

        /// <summary>
        /// Closes the connection
        /// </summary>
        void Disconnect();

        /// <summary>
        /// Executes a query on the database
        /// </summary>
        /// <param name="queryString">The query to run</param>
        /// <returns>results from the query</returns>
        DBResults Query(string queryString);

        /// <summary>
        /// Returns a value indicating if the database is connected
        /// </summary>
        /// <returns>True if connected; False otherwise</returns>
        bool IsConnected();
    }
}
