using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace MrHomer
{
    public class ErrorManager
    {
        public delegate void ErrorPrinter(Exception e);
        Queue<Exception> errs;

        #region Singleton

        private static ErrorManager instance;

        public static ErrorManager getInstance()
        {
            if (instance == null)
                instance = new ErrorManager();
            return instance;
        }

        #endregion

        private ErrorManager()
        {
            errs = new Queue<Exception>();
        }

        /// <summary>
        /// Adds a new exception
        /// </summary>
        /// <param name="e">The exception</param>
        public void Catch(Exception e)
        {
            lock (errs)
            {
                errs.Enqueue(e);
            }
        }

        /// <summary>
        /// Gets the next exception and removes it from the list
        /// </summary>
        /// <returns>The exception</returns>
        public Exception Next()
        {
            return errs.Dequeue();
        }

        /// <summary>
        /// Gets the next exception without removing it from the list
        /// </summary>
        /// <returns>The exception</returns>
        public Exception Peek()
        {
            return errs.Peek();
        }

        /// <summary>
        /// Writes all errors to a file
        /// </summary>
        /// <param name="filename">The file to which the errors will be written</param>
        public void DumpErrors(string filename)
        {
            lock (errs)
            {
                StreamWriter w = File.AppendText(filename);
                while (errs.Count > 0)
                {
                    Exception e = errs.Dequeue();
                    w.WriteLine(e.ToString());
                    w.WriteLine(e.StackTrace);
                }
            }
        }

        /// <summary>
        /// Writes all errors to a given error printer
        /// </summary>
        /// <param name="p"></param>
        public void DumpErrors(ErrorPrinter p)
        {
            lock (errs)
            {
                while (errs.Count > 0)
                {
                    Exception e = errs.Dequeue();
                    p(e);
                }
            }
        }
    }
}
