using System;
using System.Collections.Generic;
using System.Text;

namespace MrHomer.Networking
{
    public class StringListData : List<string>, IHSerializable
    {
        public StringListData()
            : base()
        {
        }

        public StringListData(IEnumerable<string> collection)
            : base(collection)
        {
        }

        public StringListData(int capacity)
            : base(capacity)
        {
        }

        #region IHSerializable Members

        /// <summary>
        /// Serializes the StringListData as a byte stream.
        /// </summary>
        /// <returns>The byte stream.</returns>
        public byte[] Serialize()
        {
            List<byte> temp = new List<byte>();

            for (int i = 0; i < this.Count; i++)
            {
                Int32 strLen = this[i].Length;
                temp.AddRange(BitConverter.GetBytes(strLen));
                temp.AddRange(Encoding.UTF8.GetBytes(this[i]));
            }
            return temp.ToArray();
        }

        /// <summary>
        /// Deserializes the byte stream.
        /// </summary>
        /// <param name="myData">The byte stream.</param>
        public void Deserialize(byte[] data)
        {
            int position = 0;
            this.Clear();

            while (position < data.Length)
            {
                Int32 strLen = BitConverter.ToInt32(data, position);
                position += sizeof(Int32);
                string r = Encoding.UTF8.GetString(data, position, strLen);
                position += strLen;
                this.Add(r);
            }
        }

        #endregion
    }
}
