﻿using System;
namespace MrHomer.Security
{
    public interface IKeyManager
    {
        /// <summary>
        /// Gets the key for an object
        /// </summary>
        /// <param name="Name">The object for which to retrieve the key</param>
        /// <param name="Details">Additional information about the object</param>
        /// <returns>The object's key</returns>
        byte[] GetKey(string Name, object[] Details);

        /// <summary>
        /// Sets the key for an object
        /// </summary>
        /// <param name="Name">The object for which to retrieve the key</param>
        /// <param name="Key">The object's key</param>
        /// <param name="Details">Additional information about the object</param>
        void SetKey(string Name, byte[] Key, object[] Details);

        /// <summary>
        /// Determines if the key manager contains the requested key
        /// </summary>
        /// <param name="Name">The name of the key</param>
        /// <param name="Details">Additional information about the object</param>
        /// <returns>Whether the key is in the key manager</returns>
        bool ContainsKey(string Name, object[] Details);

        /// <summary>
        /// Removes the key for an object
        /// </summary>
        /// <param name="Name">The object for which to retrieve the key</param>
        /// <param name="Details">Additional information about the object</param>
        void RemoveKey(string Name, object[] Details);
    }
}
