using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Networking;

namespace MrHomerDriver
{
    public class ConnectionManagerTestBackend
    {
        static private ConnectionManagerTestBackend instance;

        static public ConnectionManagerTestBackend getInstance()
        {
            if (instance == null)
                instance = new ConnectionManagerTestBackend();
            return instance;
        }

        public delegate void OutputChanged(string[] text);

        public event OutputChanged ConsoleDump;

        private List<string> output;
        public ConnectionManagerTestBackend()
        {
            output = new List<string>();
        }

        public void SetCredentials(string credentials)
        {
            output.Add("-Setting Credentials: " + credentials);
            try
            {
                ConnectionManager.getInstance().SetCredentials(new System.Text.UTF8Encoding().GetBytes(credentials));
                output.Add("--Credentials set!");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }
            ConsoleDump(output.ToArray());
        }

        public void SetRequestPacket(string requestpacket)
        {
            output.Add("-Setting Request Packet: " + requestpacket);
            try
            {
                ConnectionManager.getInstance().SetRequestPacket(new System.Text.UTF8Encoding().GetBytes(requestpacket));
                output.Add("--Request Packet set!");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }
            ConsoleDump(output.ToArray());
        }

        public void SetTimeout(int time, bool timeout)
        {
            if (timeout)
                output.Add("-Enabling timeout at " + time + " milliseconds.");
            else
                output.Add("-Disabling timeout.");
            try
            {
                ConnectionManager.getInstance().SetDropout((uint)time, timeout);
                output.Add("--Timeout set!");
            }
            catch (Exception e)
            {
                output.Add("--Epoch fail.  See? " + e.Message);
            }
            ConsoleDump(output.ToArray());

        }

        public void ListenOnPort(int port)
        {
            output.Add("-Listening on port: " + port.ToString());
            try
            {
                ConnectionManager.getInstance().ListenForConnection(port, true);
                output.Add("--It worked.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void StopListening(int port)
        {
            output.Add("-Stabbing ears on port: " + port.ToString());
            try
            {
                ConnectionManager.getInstance().StopListening(port);
                output.Add("--It worked.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void ListActive()
        {
            output.Add("-Active Connections:");
            try
            {
                foreach (string s in ConnectionManager.getInstance().GetAllActiveConnections())
                {
                    output.Add("--" + s);
                }
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void ListInactive()
        {
            output.Add("-Inactive Connections:");
            char[] trimme = {'\0'};
            try
            {
                foreach (KeyValuePair<string, byte[]> k in ConnectionManager.getInstance().GetAllInactiveConnections())
                {
                    output.Add("--" + k.Key);
                    output.Add("---" + (new System.Text.UTF8Encoding()).GetString(k.Value).Trim(trimme));
                }
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void ActivateConnection(string oldname, string newname)
        {
            output.Add("-Turning inactive connection '" + oldname + "' into active connection '" + newname + "'.");
            try
            {
                ConnectionManager.getInstance().ActivateConnection(oldname, newname);
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void ListeningPorts()
        {
            output.Add("-Listening on ports:");
            try
            {
                foreach (int s in ConnectionManager.getInstance().GetListeningPorts())
                {
                    output.Add("--" + s.ToString());
                }
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void MakeConnection(string name, string Hostname, int port)
        {
            output.Add("-Attempting Connection With:");
            output.Add("--" + Hostname);
            output.Add("--Port:" + port.ToString());
            output.Add("--With label:" + name);
            try
            {
                ConnectionManager.getInstance().MakeConnection(name, Hostname, port, true);
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());

        }

        public void KillConnection(string name)
        {
            output.Add("-Killing as active:");
            try
            {
                ConnectionManager.getInstance().KillActiveConnection(name);
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
            output.Add("-Killing as inactive:");
            try
            {
                ConnectionManager.getInstance().KillInactiveConnection(name);
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());

        }

        public void GetDropped()
        {
            output.Add("-The following connections were dropped:");
            try
            {
                foreach (string s in ConnectionManager.getInstance().GetDroppedConnections(true))
                {
                    output.Add("--" + s);
                }
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void SendData(string active, string data)
        {
            output.Add("-Sending data:");
            output.Add("--Target:" + active);
            output.Add("--Data:" + data);
            try
            {
                ConnectionManager.getInstance().SendData(active, (new System.Text.UTF8Encoding()).GetBytes(data));
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void GetData(string from)
        {
            char[] trimme = { '\0' };
            output.Add("-Reading data:");
            if(from != "")
                output.Add("--From:" + from);
            try
            {
                List<byte[]> lol = new List<byte[]>();
                if (from != "")
                    lol.AddRange(ConnectionManager.getInstance().GetData(from, true));
                else
                    lol.AddRange(ConnectionManager.getInstance().GetData(true));

                foreach (byte[] b in lol)
                {
                    output.Add("---" + (new System.Text.UTF8Encoding()).GetString(b).Trim(trimme));
                }
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }

        public void Clear()
        {
            output.Clear();
            ConsoleDump(output.ToArray());
        }

        public void GetFailed()
        {
            output.Add("-Failed Connections:");
            try
            {
                foreach (string s in ConnectionManager.getInstance().GetFailedConnections(true))
                {
                    output.Add("--" + s);
                }
                output.Add("--Done.");
            }
            catch (Exception e)
            {
                output.Add("--Epic double-fail.  See? " + e.Message);
            }

            ConsoleDump(output.ToArray());
        }
    }
}
