using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MrHomer.Database;
using MrHomerDriver.TestClasses;

namespace MrHomerDriver
{
    public partial class DatabaseTest : Form
    {
        private int countdown = 3;
        public DatabaseTest()
        {
            InitializeComponent();
        }

        private void LogMessage(string message)
        {
            txtLog.Text += message;
            txtLog.SelectionStart = txtLog.Text.Length;
            txtLog.SelectionLength = 0;
            txtLog.ScrollToCaret();
        }

        private void tmrStartup_Tick(object sender, EventArgs e)
        {
            if (countdown > 0)
            {
                LogMessage(countdown.ToString() + "... ");
                countdown--;
            }
            else
            {
                tmrStartup.Enabled = false;
                LogMessage("\nStarting\n");

                DatabaseManager dbm;
                LogMessage("Getting Database Manager...");
                try
                {
                    dbm = DatabaseManager.getInstance();
                    dbm.RegisterType("SQL", new SQL_NTAuth_Factory());
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Connecting to Database...");
                try
                {
                    dbm.ConnectDatabase("Test", "SQL", ".\\sqlexpress", "", "", "master", null);
                    //dbm.ConnectDatabase("Test", "phoenix\\sqlexpress", "", "", "master", typeof(SQL_Database_NTAuth));
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Creating a table...");
                try
                {
                    dbm.QueryDatabase("Test", "CREATE TABLE TestTable (ID INTEGER PRIMARY KEY, sample VARCHAR(50));");
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Inserting into table...");
                try
                {
                    dbm.QueryDatabase("Test", "INSERT INTO TestTable (ID, sample) VALUES (9999, 'Test Data');");
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Selecting from table...");
                try
                {
                    DBResults r = dbm.QueryDatabase("Test", "SELECT * FROM TestTable;");
                    if ((r[0, 1].ToString()) != "Test Data")
                    {
                        LogMessage("FAIL!!!\n");
                        return;
                    }
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Updating table...");
                try
                {
                    dbm.QueryDatabase("Test", "UPDATE TestTable SET sample='New Test' WHERE ID=9999;");
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Deleting from table...");
                try
                {
                    dbm.QueryDatabase("Test", "DELETE FROM TestTable WHERE ID=9999;");
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Deleting from table...");
                try
                {
                    dbm.QueryDatabase("Test", "DELETE FROM TestTable WHERE ID=9999;");
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Dropping table...");
                try
                {
                    dbm.QueryDatabase("Test", "DROP TABLE TestTable;");
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("Disconnecting from database...");
                try
                {
                    dbm.DisconnectDatabase("Test");
                }
                catch (Exception ex)
                {
                    LogMessage("FAIL!!!\n");
                    LogMessage("Exception: " + ex.Message);
                    LogMessage(ex.StackTrace);
                    return;
                }
                LogMessage("OK!\n");

                LogMessage("COMPLETE!!");
            }
        }

        private void DatabaseTest_Load(object sender, EventArgs e)
        {
            LogMessage("Starting in ");
        }
    }
}