using System;
using System.Collections.Generic;
using System.Text;
using MrHomer.Networking;
using AsyncSockets;
using System.Net.Sockets;
using System.Collections;
using MrHomer.Security;

namespace MrHomerDriver.TestClasses
{
    public class DemoAuthSystem
    {
        public delegate void AuthenticationEvent(object sender, SocketConnectEventArgs e);
        public event AuthenticationEvent AuthenticationCompleted;

        SecurityManager mgr;

        public DemoAuthSystem(SecurityManager security)
        {
            mgr = security;
        }

        /// <summary>
        /// The client performing the connection to a server above MUST call this to begin Authentication.
        /// </summary>
        /// <param name="host">The newly opened socket to the remote host</param>
        /// <param name="rHost">The identifier of the remote host</param>
        public void Authenticate(AsyncSocket host, string rHost)
        {
            //DEBUG ONLY!!!
            string sHost = "SENDER!!";

            // Step 2 -- Send challenge encrypted with P
            string message = "Authenticate " + sHost;
            byte[] plain = MakePacket(sHost, rHost, message);
            byte[] crypt = mgr.Encrypt(plain, new object[] { mgr.GetKey(rHost, null) });

            //host.BaseSocket.Send(crypt);

            // Receive:
            byte[] expected = MakePacket(sHost, rHost, "Acknowledge " + sHost);
            byte[] actual = new byte[expected.Length];

            // Step 7 -- Receive response, decrypt with Q
            //host.BaseSocket.Receive(actual, expected.Length, SocketFlags.None);
            byte[] actual_decrypted = mgr.Decrypt(actual, new object[] { mgr.GetKey("<<SELF>>", null) });

            // Step 8 -- Verify signature with P
            byte[] actual_final = mgr.Decrypt(actual_decrypted, new object[] { mgr.GetKey(rHost, null) });

            // Step 9 -- Verify
            if (actual_final.Equals(expected)) throw new Exception("Authentication Failed");

            // Step 10 -- Generate new key, sign, and send
            // DEBUG ONLY!!!
            byte[] newKey = new byte[] { 2, 2, 2, 2 };

            plain = MakePacket(sHost, rHost, Encoding.UTF8.GetChars(newKey).ToString());
            crypt = mgr.Encrypt(plain, mgr.GetKey("<<SELF>>", new object[] { mgr.GetKey("<<SELF>>", null) }));
            //host.BaseSocket.Send(crypt);

            // If we get here, authentication passed on client end
        }

        private byte[] MakePacket(string src, string dest, string message)
        {
            StringData n =  new StringData();
            n.Data = message;

            Packet p = new Packet(src, dest, "Auth", n);
            return p.Serialize();
        }

        /// <summary>
        /// The host system MUST register this as the callback for the AcceptedConnection event of
        /// the listening socket.  It will raise the AuthenticationCompleted event when finished.
        /// </summary>
        /// <param name="sender">The listening socket</param>
        /// <param name="e">Arguments from the response</param>
        public void AuthenticateServer(object sender, SocketConnectEventArgs e)
        {
            string sHost = "RECEIVER!!!";

            AsyncSocket r = e.ConnectedClient;
            ((AsyncSocket)sender).Accept(); // Prepare for the next attempt

            // Step 3 -- Receive packet
            byte[] buffer = new byte[1024];
            ArrayList bigBuffer = new ArrayList();
            do
            {
                //r.BaseSocket.Receive(buffer);
                bigBuffer.AddRange(buffer);
            } while (r.Available > 0);

            byte[] received = (byte[])bigBuffer.ToArray(typeof(byte));

            /// Step 4 -- Decrypt message with Q
            byte[] received_decrypted = mgr.Decrypt(received, new object[] { mgr.GetKey("<<SELF>>", null) });
            string got = PacketBody(received_decrypted);

            //TODO: Handle failure
            if (!got.StartsWith("Authenticate ")) return;

            string rHost = got.Substring(13);

            // Step 5 -- Form response and sign with Q
            string response = "Acknowledge " + rHost;
            byte[] plain = MakePacket(sHost, rHost, response);
            byte[] crypt = mgr.Encrypt(plain, new object[] { mgr.GetKey("<<SELF>>", null) });

            // Step 6 -- Encrypt with P and send
            byte[] crypt2 = mgr.Encrypt(plain, new object[] { mgr.GetKey(rHost, null) });

            //r.BaseSocket.Send(crypt2);

            //TODO: Figure out how to decode this
            bigBuffer.Clear();

            do
            {
                //r.BaseSocket.Receive(buffer);
                bigBuffer.AddRange(buffer);
            } while (r.Available > 0);

            byte[] newKey = (byte[])bigBuffer.ToArray(typeof(byte));
            byte[] decrypt_key = mgr.Decrypt(newKey, new object[] { mgr.GetKey(rHost, null) });
            mgr.SetKey(rHost, new object[] { Encoding.UTF8.GetBytes(PacketBody(decrypt_key)) });

            // If we get here, authentication passed on server end
            AuthenticationCompleted(sender, e);
        }

        private string PacketBody(byte[] data)
        {
            Packet p = new Packet(data);

            StringData d = (StringData)p.Body;

            return d.Data;
        }
    }
}
