using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using MrHomer.Database;

namespace MrHomerDriver.TestClasses
{
    public class SQL_Database : IDatabase
    {
        protected SqlConnection conn;

        public SQL_Database() { }

        #region IDatabase Members

        public virtual bool Connect(string dataSource, string username, string password, string database)
        {
            SqlConnectionStringBuilder n = new SqlConnectionStringBuilder();

            n.DataSource = dataSource;
            n.IntegratedSecurity = false;
            n.Password = password;
            n.UserID = username;
            n.InitialCatalog = database;

            try
            {
                conn = new SqlConnection(n.ConnectionString);
                conn.Open();
            }
            catch
            {
                conn = null;
                return false;
            }

            return true;
        }

        public void Disconnect()
        {
            conn.Close();
        }

        public DBResults Query(string queryString)
        {
            SqlCommand n = new SqlCommand(queryString, conn);
            SqlDataReader r = n.ExecuteReader();
            DBResults res = null;

            try
            {
                string[] keys = new string[r.VisibleFieldCount];
                List<object[]> data = new List<object[]>();

                for (int i = 0; i < r.VisibleFieldCount; i++)
                {
                    keys[i] = r.GetName(i);
                }

                //do
                while (r.Read())
                {
                    object[] tmp = new object[r.VisibleFieldCount];

                    r.GetSqlValues(tmp);

                    data.Add(tmp);
                } //while (r.Read());

                res = new DBResults(keys, data.ToArray());
            }
            catch
            {
                // Do nothing?
            }
            finally
            {
                r.Close();
            }

            return res;
        }

        public bool IsConnected()
        {
            return conn != null;
        }

        #endregion
    }
}
