using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Communication;
using MrHomer;
using MrHomer.Networking;
using DuffBeer.Sensors;
using System.Threading;
using MrHomer.Security;
using DuffBeer;
using DuffBeerDriver.FakePlugins;
using DuffBeer.Security.Cryptography;
using DuffBeer.Messages.MessageTypes;
using MrHomer.Plugins;

namespace On_Site_Server
{
    /// <summary>
    /// This class holds the logic for the methods needed by the On-Site Server
    /// </summary>
    class Core
    {
        #region Variables
        private static volatile CommunicationManager cm;
        private static ErrorManager em;
        private static ConnectionManager mm;
        private static SensorManager sm;
        private static PluginManager pm;
        public static MainForm.DelayedMessage Logger;
        private static Thread t;
        #endregion
        #region Helpers

        private static bool FilterFinder(FilterInformation f)
        {
            return f.Name != "Composite Data Filter";
        }

        private static void Log(string message)
        {
            Logger(message);
        }

        private static void ErrorLog(Exception e)
        {
            Log(e.ToString() + "\n");
        }

        private static void MsgThreader()
        {
            while (cm.HasEvents)
            {
                try
                {
                    cm.DoEvents();
                    em.DumpErrors(new ErrorManager.ErrorPrinter(Core.ErrorLog));
                    Thread.Sleep(100);
                }
                catch (ThreadAbortException)
                {
                    return;
                }
            }
            Log("Event Loop Finished");
        }
        #endregion
        #region Methods
        /// <summary>
        /// Runs the Application
        /// </summary>
        public static void Run()
        {
            Log("Welcome to the MrHomer On-site Server Program");
            performInit();
            Log("Running on Port: " + Brewery.getInstance().Config("Port"));
            Log("Initialization complete..." + "\n");
        }

        private static void performInit()
        {
            SecurityManager.getInstance(new SuperAuth(), new NRotN_Crypto(), new SuperKey());
            Brewery.getInstance();
            cm = CommunicationManager.getInstance();
            mm = ConnectionManager.getInstance();
            em = ErrorManager.getInstance();
            sm = SensorManager.getInstance();
            pm = PluginManager.getInstance();

            performSensorStartup();
            mm.NotifyOfClientConnection += new ConnectionManager.ConnectNotification(notifyOfEvent);
            SensorDataRequestMessageHandler.RequestNotification += new SensorDataRequestMessageHandler.SensorDataRequestNotification(notifyOfEvent);
            SensorListRequestMessageHandler.RequestNotification += new SensorListRequestMessageHandler.SensorListRequestNotification(notifyOfEvent);
            mm.NotifyOfClientDisconnection += new ConnectionManager.DisconnectNotification(notifyOfEvent);
            DatabaseCleanupFilter.CleanDataLoggedIntoDBNotification += new DatabaseCleanupFilter.CleanDataLoggedNotification(notifyOfEvent);
            DatabaseFilter.DataLoggedIntoDBNotification += new DatabaseFilter.DataLoggedNotification(notifyOfEvent);

            t = new Thread(new ThreadStart(Core.MsgThreader));
            t.Name = "Events Loop";
            t.Start();
        }

        /// <summary>
        /// starts up test sensor
        /// </summary>
        private static void performSensorStartup() 
        {
            Log("Setting up local sensors...");

            //string sName = sm.GetSensorTypeInformation()[0].Name;
            //List<FilterInformation> f = sm.GetFilterTypeInformation().FindAll(new Predicate<FilterInformation>(FilterFinder));
            //string fName = f[0].Name;       //database cleanup filter
            //string fName2 = f[1].Name;      //database filter
            
            //instantiate one PV sensor and add the filters to it
            sm.InstantiateSensor("PVSensor1", "Photo Voltaic", "PV Sensor", 5000);//900000);
            sm.AddFilter("PVSensor1", "Database Cleanup Filter", "Database Cleanup Filter");
            sm.AddFilter("PVSensor1", "Database Filter 1", "Database Filter");

            //start the first PV Sensor
            if (sm.StartSensor("PVSensor1"))
                Log("PVSensor1 Started.");
            else
                Log("Couldn't start PVSensor1...");

            //sm.InstantiateSensor("TestSensor1", "Fake", "Test Sensor", 1000);
            //sm.AddFilter("TestSensor1", "Test Filter 1", "Test Data Filter");
            //Log("Started TestSensor1 " + sm.StartSensor("TestSensor1"));
            
        }
        #endregion

        #region Callbacks

        /// <summary>
        /// Callback function which displays the contents of the message raised by an event
        /// </summary>
        /// <param name="msg">The message to write</param>
        private static void notifyOfEvent(string msg)
        {
            Log(msg + "\n");
        }
        
        #endregion
    }
}
