using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace On_Site_Server
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            Thread.CurrentThread.Name = "Main Form";
        }

        internal delegate void DelayedMessage(string message);
        
        /// <summary>
        /// Logs a message to the console
        /// </summary>
        /// <param name="message"></param>
        private void LogMessage(string message)
        {
            if (Console.IsDisposed) return;
            if (Console.InvokeRequired)
            {
                Console.BeginInvoke(new DelayedMessage(LogMessage), message);
            }
            else
            {
                Console.Text += DateTime.Now.ToString() + " > " + message + "\n";
                Console.SelectionStart = Console.Text.Length;
                Console.SelectionLength = 0;
                Console.ScrollToCaret();
            }
        }

        private string ArrayToString(string name, Array a)
        {
            string result = name + " { ";
            for (int i = 0; i < a.Length - 1; i++)
                result += a.GetValue(i).ToString() + ", ";

            result += a.GetValue(a.Length - 1) + " }\n";

            return result;
        }

        private bool ArrayEquals(Array a, Array b)
        {
            if (a == null || b == null) return false;
            if (a.Length != b.Length) return false;

            try
            {
                for (int i = 0; i < a.Length; i++)
                    if (!a.GetValue(i).Equals(b.GetValue(i)))
                        return false;
            }
            catch
            {
                return false;
            }

            return true;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            Core.Logger = LogMessage;
            Thread t = new Thread(new ThreadStart(Core.Run));
            t.Name = "Main";
            t.Start();
        }

        private void MainForm_Closing(object sender, FormClosingEventArgs e)
        {
            //this program should not be closable
            e.Cancel = true;
        }
    }
}