using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Sensors.Interfaces;
using MrHomer.Plugins;
using DuffBeer.Sensors.Utility;
using DuffBeer;
using MrHomer.Networking;
using DuffBeer.Communication;
using MrHomer.Security;
using DuffBeer.Messages.MessageTypes;
using DuffBeer.Sensors;

namespace On_Site_Server
{
    /// <summary>
    ///Adding this filter to a sensor allows the sensor to send real-time data reports
    ///to all connected requesting applications.
    /// </summary>
    [MrHomerPlugin("Name", "Real Time Reporting Filter")]
    [MrHomerPlugin("Description", "Reports data back to a requesting application in real time")]
    class RealTimeReportingFilter :IDataFilter
    {
        #region IDataFilter Members

        private Sensor sensor;
        public Sensor associatedSensor
        {
            get
            {
                return sensor;
            }
            set
            {
                sensor = value;
            }
        }

        public object Filter(object data)
        {
            DataReading r = data as DataReading;

            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(RealTimeReportingResponseMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            //send a message to each connected client with real-time data in it
            foreach (string conn in ConnectionManager.getInstance().GetAllActiveConnections())
            {
                Packet p = new Packet(Brewery.getInstance().UPRN, conn, subject, new StringData(r.SensorID + " : " + r.Value ));
                ITask task = new SendDataTask();
                task.InitializeTask(conn, new object[] { p, SecurityManager.getInstance().GetKey(conn, new object[] { }), new object[] { } });
                CommunicationManager.getInstance().AddTask(task);
            }
            return r;
        }

        public byte[] SerializeConfiguration()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void DeserializeConfiguration(byte[] data)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public Type InputType
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public Type OutputType
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        private int dbid;
        public int DBID
        {
            get
            {
                return dbid; 
            }
            set
            {
                dbid = value;
            }
        }

        #endregion

        #region IPlugin Members

        public string Author
        {
            get { return "Nathan Roy"; }
        }

        public string Version
        {
            get { return "V 0.1"; }
        }

        public string Label
        {
            get { return "Provides real time-updates back to the Remote Requesting Application"; }
        }

        #endregion
    }
}
