using System;
using System.Collections.Generic;
using System.Text;
using DuffBeer.Database.StructureTypes;
using DuffBeer.Database;
using MrHomer.Database;

namespace Remote_Requesting_Application
{
    /// <summary>
    /// This class emcompasses a entry into the Database table called GISData in our MrHomer Database.
    /// It allows you to create an entry, save entries to the DB, and find entries.  This class is meant
    /// to be used for logging incoming data in the Remote Requesting Application to a database based on
    /// the UPRN it came from.  This should allow for tagging UPRN specific data onto a GIS map.
    /// </summary>
    class GISDataTableEntry : TableEntry
    {
        static public string UPRNColumn
        {
            get { return "UPRN"; }
        }

        static public string SensorNameColumn
        {
            get { return "SensorName"; }
        }

        static public string ReadingColumn
        {
            get { return "Reading"; }
        }

        static public string TimestampColumn
        {
            get { return "Timestamp"; }
        }

        public string UPRN
        {
            get { return myTableRef.Row[UPRNColumn].ToString(); }
            set
            {
                string s = value.ToString();
                if(s.StartsWith("UPRN"))
                {
                    s = s.Remove(0, "UPRN".Length);
                    myTableRef.Row[UPRNColumn] = s ;
                }
                else
                    myTableRef.Row[UPRNColumn] = value.ToString(); 
            }
        }

        public string SensorName
        {
            get { return myTableRef.Row[SensorNameColumn].ToString(); }
            set { myTableRef.Row[SensorNameColumn] = value.ToString(); }
        }

        public double Reading
        {
            get { return double.Parse(myTableRef.Row[ReadingColumn]); }
            set { myTableRef.Row[ReadingColumn] = value.ToString(); }
        }

        public DateTime Timestamp
        {
            get { return DateTime.Parse(myTableRef.Row[TimestampColumn]); }
            set { myTableRef.Row[TimestampColumn] = value.ToString(); }
        }

        public GISDataTableEntry()
            : base(DuffBeerDatabaseConfig.GISDataTable)
        {
        }

        public GISDataTableEntry(int RowID)
            : this()
        {
            myTableRef.Find("ID", RowID.ToString());
        }

        public GISDataTableEntry(string UPRN, string SensorName, double Reading, DateTime Timestamp)
            : this()
        {
            myTableRef.AddRow();
            this.UPRN = UPRN;
            this.SensorName = SensorName;
            this.Reading = Reading;
            this.Timestamp = Timestamp;
        }

        public static GISDataTableEntry Find(string Column, string Value)
        {
            DataTable t = new DataTable(DatabaseManager.getInstance().FindDatabase(DuffBeerDatabaseConfig.DatabaseName), DatabaseManager.getInstance().GetQueryBuilder(DuffBeerDatabaseConfig.DatabaseName), DuffBeerDatabaseConfig.GISDataTable, "ID");
            t.Sort(TimestampColumn, DataTable.SortOrder.Descending);
            if (t.Find(Column, Value))
                return new GISDataTableEntry(int.Parse(t.Row["ID"]));
            else
                return null;
        }
    }
 }

