using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Text.RegularExpressions;
using DuffBeer.Messages.MessageTypes;
using MrHomer.Networking;
using MrHomer.Plugins;
using DuffBeer.Communication;
using MrHomer.Security;
using DuffBeer;

namespace Remote_Requesting_Application
{
    public partial class MainForm : Form
    {
        public string Connection = "BYB Connection";
        Core core;
        #region Constructors
        public MainForm()
        {
            InitializeComponent();
            RealTimeReportingResponseMessageHandler.RealTimeDataArrivedNotification += new RealTimeReportingResponseMessageHandler.RealTimeDataReceivedNotification(displayRealTimeData);
        }
         internal delegate void DelayedMessage(string message);
        #endregion
        #region Methods

        /// <summary>
        /// Logs a message to the console
        /// </summary>
        /// <param name="message"></param>
        private void LogMessage(string message)
        {
            if (Console.IsDisposed) return;
            if (Console.InvokeRequired)
            {
                Console.BeginInvoke(new DelayedMessage(LogMessage), message);
            }
            else
            {
                Console.Text += DateTime.Now.ToString() + " > " + message + "\n";
                Console.SelectionStart = Console.Text.Length;
                Console.SelectionLength = 0;
                Console.ScrollToCaret();
            }
        }

        private string ArrayToString(string name, Array a)
        {
            string result = name + " { ";
            for (int i = 0; i < a.Length - 1; i++)
                result += a.GetValue(i).ToString() + ", ";

            result += a.GetValue(a.Length - 1) + " }\n";

            return result;
        }

        private bool ArrayEquals(Array a, Array b)
        {
            if (a == null || b == null) return false;
            if (a.Length != b.Length) return false;

            try
            {
                for (int i = 0; i < a.Length; i++)
                    if (!a.GetValue(i).Equals(b.GetValue(i)))
                        return false;
            }
            catch
            {
                return false;
            }

            return true;
        }
        /// <summary>
        /// Requests a real-time data feed from the server for the sensor given
        /// </summary>
        /// <param name="sensor"></param>
        public void RequestDataFeedFor(string sensor)
        {
            LogMessage("Requesting Real-Time Data Feed...");
            //if we have not removed the message handler callback, put in a request for real-time data
            if (RealTimeReportingResponseMessageHandler.RealTimeDataArrivedNotification != null)
            {
                string subject = "";
                foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(RealTimeReportingRequestMessageHandler)))
                {
                    if (n.Name == "Subject")
                        subject = n.Value;
                }

                Packet p = new Packet(Brewery.getInstance().UPRN, Connection, subject, new StringData(sensor));
                ITask task = new SendDataTask();
                task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
                CommunicationManager.getInstance().AddTask(task);

                LogMessage("Request Sent...\n");
            }
            else //otherwise, we can just re-tap into the data feed that is already there
                RealTimeReportingResponseMessageHandler.RealTimeDataArrivedNotification += new RealTimeReportingResponseMessageHandler.RealTimeDataReceivedNotification(displayRealTimeData);
        }

        /// <summary>
        /// Removes a real-time data feed from the sensor
        /// </summary>
        /// <param name="sensor">the sensor to remove from</param>
        public void RemoveDataFeedFor(string sensor)
        {
            LogMessage("Removing Real-Time Data Feed for: " + sensor + "...");
            liveFeedBox.Text = "";
            RealTimeReportingResponseMessageHandler.RealTimeDataArrivedNotification -= new RealTimeReportingResponseMessageHandler.RealTimeDataReceivedNotification(displayRealTimeData);
            
            string subject = "";
            foreach (MrHomerPluginAttribute n in PluginManager.getInstance().getAttributes(typeof(RealTimeReportingRemoveRequestMessageHandler)))
            {
                if (n.Name == "Subject")
                    subject = n.Value;
            }

            Packet p = new Packet(Brewery.getInstance().UPRN, Connection, subject, new StringData(sensor));
            ITask task = new SendDataTask();
            task.InitializeTask(Connection, new object[] { p, SecurityManager.getInstance().GetKey(Connection, new object[] { }), new object[] { } });
            CommunicationManager.getInstance().AddTask(task);
        }
        #endregion
        #region Event Handlers

        private void MainForm_Load(object sender, EventArgs e)
        {
            core = new Core(this);
            Core.Logger = LogMessage;

            Thread t = new Thread(new ThreadStart(core.Run));
            t.Name = "Main";
            t.Start();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            //TODO : ALLOW THE STUPID PROGRAM TO CLOSE
            core.CleanUpAndQuit();
        }

        /// <summary>
        /// Connects to the on-site server at the address specified in remoteIPBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnConnect_Click(object sender, EventArgs e)
        {
            //Create a regular expression to test if the IP address is in the correct format
            //of xxx.xxx.xxx.xxx
            string pattern = @"[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}";
                            //@"\d\d?\d?\.\d\d?\d?\.\d\d?\d?\.\d\d?\d?";

            if (Regex.IsMatch(remoteIPBox.Text, pattern))
            {
                LogMessage("Trying to connect to computer at: " + remoteIPBox.Text);
                if (core.Connect(remoteIPBox.Text))
                {
                    disconBtn.Enabled = true;
                    getSensorsBtn.Enabled = true;
                    sensorListBox.Enabled = true;
                }
            }

            else
            {
                LogMessage("IP Address was not in the correct format");
            }
        }

        /// <summary>
        /// Requests the Usage data from the sensors from the on-site server
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getDataBtn_Click(object sender, EventArgs e)
        {
            LogMessage("Requesting Sensor Data for " + sensorListBox.SelectedItem.ToString() + " from the On-Site Server...");
            //Thread.Sleep(0);

            Dictionary<string, string> results = core.RequestSensorData(sensorListBox.SelectedItem.ToString());

            LogMessage("Data Returned: ");
            foreach (KeyValuePair<string, string> kvp in results)
            {
                LogMessage(kvp.Key + ": " + kvp.Value);
            }
            LogMessage("End Data Returned\n");

            //E TERM TODO: This code logs data into the GISData table.  You may want to add code to replace the 
            //latest entry in the table from the sensor requested with this new one or something similar to that.  
            //GISDataTableEntry gisEntry = new GISDataTableEntry(results["UPRN"], results["Name"], double.Parse(results["Reading"]), DateTime.Parse(results["Timestamp"]));
            //gisEntry.Save();
                
        }

        /// <summary>
        /// Handles the clicking of the disconnect button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void disconBtn_Click(object sender, EventArgs e)
        {
            core.Disconnect();
            disconBtn.Enabled = false;
            getDataBtn.Enabled = false;
            getSensorsBtn.Enabled = false;
            requestFeedBtn.Enabled = false;
            removeDataFeedBtn.Enabled = false;
            sensorListBox.Items.Clear();
            sensorListBox.Text = "";
            liveFeedBox.Text = "";
        }

        /// <summary>
        /// Handles the clicking of the request sensor list button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void getSensorsBtn_Click(object sender, EventArgs e)
        {
            LogMessage("Requesting sensor list from server...");
            List<String> ret = core.RequestSensorList();

            //first clear the previous sensor list
            sensorListBox.Items.Clear();

            foreach (string s in ret)
            {
                sensorListBox.Items.Add(s);
            }
            sensorListBox.SelectedIndex = 0;

            getDataBtn.Enabled = true;
            requestFeedBtn.Enabled = true;
        }

        /// <summary>
        /// Handles the clicking of the request Feed Button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void requestFeedBtn_Click(object sender, EventArgs e)
        {
            RequestDataFeedFor(sensorListBox.SelectedItem.ToString());
            removeDataFeedBtn.Enabled = true;
        }

        private void removeDataFeedBtn_Click(object sender, EventArgs e)
        {
            RemoveDataFeedFor(sensorListBox.SelectedItem.ToString());
            removeDataFeedBtn.Enabled = false;
            liveFeedBox.Text = "";
        }

    #endregion
        #region Callbacks
        
        /// <summary>
        /// Called upon receipt of real-time data readouts from the server
        /// </summary>
        /// <param name="f">where the data is from</param>
        /// <param name="d">the data received</param>
        public void displayRealTimeData(string d)
        {
            if (liveFeedBox.InvokeRequired)
            {
                liveFeedBox.BeginInvoke(new DelayedMessage(displayRealTimeData), d);
            }
            else
            {
                liveFeedBox.Text = "\n\n" + d + " kWh";
             
            }
        }

        #endregion

    }
}